"""
Core Research Integration Module
Implements multi-source academic paper integration with translation support.
"""
import os
import json
import aiohttp
import requests
from datetime import datetime, timezone
from typing import Dict, List, Optional, Any
from pathlib import Path
import traceback
import asyncio
import time

from .security.research_vault import ResearchVault
from .translation_pipeline import TranslationPipeline
from .hf_integration import HFIntegration

class ResearchCore:
    def __init__(self):
        self.vault = ResearchVault()
        self.translator = TranslationPipeline()
        self.hf = HFIntegration(self.vault)
        
        # Initialize cache
        self.cache_dir = Path(__file__).parent / "cache"
        self.cache_dir.mkdir(exist_ok=True)
        
        # Initialize services
        self.services = {}
        self._init_services()
        
        # Initialize search queue and workers
        self.search_queue = asyncio.Queue()
        self.workers = [
            asyncio.create_task(self._search_worker())
            for _ in range(4)
        ]
        
        # Initialize progress tracking
        self.progress = {
            'total': 0,
            'completed': 0,
            'failed': 0,
            'last_update': datetime.now(timezone.utc).timestamp()
        }
        
    def _init_services(self):
        """Initialize available research services"""
        services = {
            "CORE": self._init_core,
            "SCIENCEOPEN": self._init_scienceopen,
            "HUGGINGFACE": self._init_huggingface,
            "HAL": self._init_hal
        }
        
        for name, init_func in services.items():
            try:
                self.services[name] = init_func()
                print(f"Initialized {name} service")
            except Exception as e:
                print(f"Failed to initialize {name} service: {e}")
                self.services[name] = None
                
    async def fetch_data(self, source: str, params: Dict[str, Any] = None) -> Dict[str, Any]:
        """Fetch data from various sources"""
        try:
            if source == "census.gov":
                return await self._fetch_census_data(params)
            elif source == "fred.stlouisfed.org":
                return await self._fetch_fred_data(params)
            elif source == "bloomberg.com":
                return await self._fetch_bloomberg_data(params)
            elif source == "statista.com":
                return await self._fetch_statista_data(params)
            elif source == "web_search":
                return await self._fetch_web_search(params)
            else:
                raise ValueError(f"Unsupported data source: {source}")
                
        except Exception as e:
            error_context = {
                "timestamp": datetime.now(timezone.utc).isoformat(),
                "source": source,
                "params": params,
                "stack_trace": traceback.format_exc(),
                "retry_count": 0
            }
            error_context['vectorized'] = self.hf_service.embed_text(
                error_context['stack_trace'],
                model='sentence-transformers/all-mpnet-base-v2'
            )
            self._store_error_context(error_context)
            print(f"Warning: Failed to fetch data from {source}: {str(e)}")
            return {}
            
    async def _fetch_census_data(self, params: Dict[str, Any] = None) -> Dict[str, Any]:
        """Fetch retail data from Census Bureau"""
        # Implement actual API call when credentials are available
        return {
            "source": "census.gov",
            "status": "simulated",
            "retail_sales": {
                "q4_2022": {
                    "total": "$1.8T",
                    "growth": "3.2%",
                    "categories": {
                        "electronics": "+5.4%",
                        "clothing": "+2.1%",
                        "food": "+4.3%"
                    }
                }
            }
        }
        
    async def _fetch_fred_data(self, params: Dict[str, Any] = None) -> Dict[str, Any]:
        """Fetch economic indicators from FRED"""
        # Implement actual API call when credentials are available
        return {
            "source": "fred.stlouisfed.org",
            "status": "simulated",
            "indicators": {
                "q4_2022": {
                    "gdp_growth": "2.9%",
                    "inflation": "7.1%",
                    "unemployment": "3.5%"
                }
            }
        }
        
    async def _fetch_bloomberg_data(self, params: Dict[str, Any] = None) -> Dict[str, Any]:
        """Fetch market data from Bloomberg"""
        # Implement actual API call when credentials are available
        return {
            "source": "bloomberg.com",
            "status": "simulated",
            "market_data": {
                "q4_2022": {
                    "sp500": "+7.1%",
                    "retail_index": "+4.2%",
                    "consumer_confidence": "108.3"
                }
            }
        }
        
    async def _fetch_statista_data(self, params: Dict[str, Any] = None) -> Dict[str, Any]:
        """Fetch industry statistics from Statista"""
        # Implement actual API call when credentials are available
        return {
            "source": "statista.com",
            "status": "simulated",
            "retail_trends": {
                "q4_2022": {
                    "ecommerce_share": "21.2%",
                    "mobile_commerce": "+18.4%",
                    "store_foot_traffic": "-2.1%"
                }
            }
        }
        
    async def _fetch_web_search(self, params: Dict[str, Any] = None) -> Dict[str, Any]:
        """Fetch web search results"""
        # Implement actual API call when credentials are available
        return {
            "source": "web_search",
            "status": "simulated",
            "results": [
                {"title": "Example result 1", "url": "https://example.com"},
                {"title": "Example result 2", "url": "https://example.com"},
                {"title": "Example result 3", "url": "https://example.com"}
            ]
        }
        
    def _init_core(self) -> Dict[str, Any]:
        """Initialize CORE.ac.uk API client"""
        try:
            return {
                "endpoint": "https://api.core.ac.uk/v3/",
                "headers": self.vault.generate_headers("CORE"),
                "rate_limit": (1000, "day"),
                "search_endpoint": "search/works"
            }
        except ValueError:
            print("CORE API key not found, service will be disabled")
            return None
            
    def _init_scienceopen(self) -> Dict[str, Any]:
        """Initialize ScienceOpen API client"""
        try:
            return {
                "endpoint": "https://www.scienceopen.com/api/v2/",
                "oauth": self.vault.get_oauth_credentials("SCIENCEOPEN"),
                "search_endpoint": "records/search"
            }
        except ValueError:
            print("ScienceOpen credentials not found, service will be disabled")
            return None
            
    def _init_hal(self) -> Dict[str, Any]:
        """Initialize HAL API client"""
        try:
            return {
                "endpoint": "https://api.archives-ouvertes.fr/search/",
                "oauth": self.vault.get_oauth_credentials("HAL"),
                "translation": {
                    "provider": "deepl",
                    "fallback": "opus_mt",
                    "context_preserve": ["math", "finance"]
                }
            }
        except ValueError:
            print("HAL credentials not found, service will be disabled")
            return None
            
    def _init_huggingface(self) -> Dict[str, Any]:
        """Initialize HuggingFace API client"""
        try:
            return {
                "endpoint": "https://huggingface.co/api/v1",
                "headers": self.vault.generate_headers("HUGGINGFACE"),
                "rate_limit": (500, "hour"),
                "search_endpoints": {
                    "models": "/models",
                    "datasets": "/datasets"
                }
            }
        except ValueError:
            print("HuggingFace API key not found, service will be disabled")
            return None
            
    async def search_papers(self, query: str, sources: Optional[List[str]] = None) -> Dict[str, List[Dict]]:
        """Search papers across specified sources"""
        sources = sources or list(self.services.keys())
        results = {}
        
        for source in sources:
            if source not in self.services or self.services[source] is None:
                continue
                
            config = self.services[source]
            
            if source == "CORE":
                results[source] = await self._search_core(query)
            elif source == "SCIENCEOPEN":
                results[source] = await self._search_scienceopen(query)
            elif source == "HAL":
                results[source] = await self._search_hal(query)
        
        return results
    
    async def _search_core(self, query: str) -> List[Dict]:
        """Search CORE papers with API key auth"""
        endpoint = f"{self.services['CORE']['endpoint']}{self.services['CORE']['search_endpoint']}"
        headers = self.services['CORE']['headers']
        
        response = await self.session.get(
            endpoint,
            headers=headers,
            params={"q": query, "limit": 100}
        )
        
        if response.status_code == 200:
            return self._normalize_core_results(response.json())
        return []
    
    async def _search_scienceopen(self, query: str) -> List[Dict]:
        """Search ScienceOpen with OAuth"""
        config = self.services["SCIENCEOPEN"]
        endpoint = f"{config['endpoint']}{config['search_endpoint']}"
        
        # Ensure valid OAuth token
        if not hasattr(self, '_scienceopen_token'):
            await self._refresh_scienceopen_token()
        
        response = await self.session.get(
            endpoint,
            headers={"Authorization": f"Bearer {self._scienceopen_token}"},
            params={"q": query, "limit": 100}
        )
        
        if response.status_code == 200:
            return self._normalize_scienceopen_results(response.json())
        return []
    
    async def _search_hal(self, query: str) -> List[Dict]:
        """Search HAL with translation support"""
        config = self.services["HAL"]
        endpoint = f"{config['endpoint']}?q={query}&wt=json"
        
        response = await self.session.get(endpoint)
        
        if response.status_code == 200:
            results = response.json()
            return await self._process_hal_results(results)
        return []
    
    async def _process_hal_results(self, results: Dict) -> List[Dict]:
        """Process and translate HAL results"""
        processed = []
        
        for doc in results.get('response', {}).get('docs', []):
            # Check if translation needed
            if doc.get('language_s') == 'fr':
                translated_doc = await self.translate_content(doc.get('title_s', ''), source_lang='fr', target_lang='en')
                doc['title_en'] = translated_doc
                translated_doc = await self.translate_content(doc.get('abstract_s', ''), source_lang='fr', target_lang='en')
                doc['abstract_en'] = translated_doc
            
            processed.append(self._normalize_hal_result(doc))
        
        return processed
    
    async def translate_content(self, content: str, source_lang: str = "fr", target_lang: str = "en") -> str:
        """Translate content with fallback options"""
        try:
            return await self.translator.translate(content, source_lang, target_lang)
        except Exception as e:
            print(f"Translation failed: {e}")
            return content  # Return original content if translation fails
    
    async def _search_worker(self):
        while True:
            task = await self.search_queue.get()
            try:
                await self._process_search(task)
            finally:
                self.search_queue.task_done()

    async def parallel_search(self, queries: List[str]):
        for query in queries:
            await self.search_queue.put(query)
        await self.search_queue.join()

    async def _process_search(self, query: str):
        max_retries = 3
        for attempt in range(max_retries):
            try:
                result = await self.fetch_data(
                    source='web_search',
                    params={'query': query}
                )
                await self.data_lake.store_search_result(query, result)
                self.progress['completed'] += 1
                return
            except Exception as e:
                if attempt == max_retries - 1:
                    self.progress['failed'] += 1
                    self.logger.error(f"Search failed for {query}: {str(e)}")
                else:
                    print(f"Warning: Failed to fetch data from web_search (attempt {attempt+1}/{max_retries}): {str(e)}")
    
    def get_progress(self) -> dict:
        return {
            'percentage': (self.progress['completed'] / self.progress['total']) * 100,
            'failed': self.progress['failed'],
            'active_workers': sum(not w.done() for w in self.workers)
        }
    
    def _normalize_core_results(self, data: Dict) -> List[Dict]:
        """Normalize CORE API results to common format"""
        normalized = []
        for item in data.get("results", []):
            normalized.append({
                "title": item.get("title"),
                "abstract": item.get("abstract"),
                "authors": item.get("authors", []),
                "published_date": item.get("published_date"),
                "doi": item.get("doi"),
                "source": "CORE"
            })
        return normalized
    
    def _normalize_scienceopen_results(self, data: Dict) -> List[Dict]:
        """Normalize ScienceOpen API results"""
        normalized = []
        for record in data.get('records', []):
            normalized.append({
                "id": record.get('id'),
                "title": record.get('title'),
                "abstract": record.get('abstract'),
                "authors": record.get('authors', []),
                "published_date": record.get('published'),
                "source": "SCIENCEOPEN",
                "url": record.get('url')
            })
        return normalized
    
    def _normalize_hal_result(self, doc: Dict) -> Dict:
        """Normalize HAL API results"""
        return {
            "id": doc.get('docid'),
            "title": doc.get('title_en', doc.get('title_s')),
            "abstract": doc.get('abstract_en', doc.get('abstract_s')),
            "authors": doc.get('author_s', []),
            "published_date": doc.get('publicationDate_s'),
            "source": "HAL",
            "url": doc.get('uri_s'),
            "original_language": doc.get('language_s')
        }
    
    async def search_ai_templates(
        self,
        task: str,
        framework: str = "pytorch",
        include_code: bool = True,
        min_downloads: int = 1000,
        min_likes: int = 50
    ) -> List[Dict]:
        """Search for AI model templates with quantitative focus"""
        try:
            # Search quant-specific models first
            models = await self.hf.search_quant_models(
                task=task,
                framework=framework,
                min_downloads=min_downloads,
                min_likes=min_likes
            )
            
            if not include_code:
                return models
            
            # Generate code templates and evaluate models
            templates = []
            for model in models[:5]:  # Limit to top 5 models
                template = await self.hf.generate_code_template(
                    model_id=model["id"],
                    task=task,
                    framework=framework
                )
                
                if template:
                    # Get model evaluation
                    evaluation = await self.hf.evaluate_model_metrics(
                        model_id=model["id"],
                        task=task
                    )
                    
                    templates.append({
                        "model": model,
                        "template": template,
                        "evaluation": evaluation,
                        "github_links": template.get("github_links", [])
                    })
            
            return templates
        except Exception as e:
            print(f"Error searching AI templates: {e}")
            return []
    
    async def evaluate_model_for_strategy(
        self,
        model_id: str,
        task: str,
        strategy_data: Optional[Dict] = None
    ) -> Dict:
        """Evaluate model suitability for trading strategy"""
        try:
            # Get model evaluation
            evaluation = await self.hf.evaluate_model_metrics(
                model_id=model_id,
                task=task,
                sample_data=strategy_data
            )
            
            # Add trading-specific assessments
            trading_assessment = {
                "latency_suitable": self._assess_trading_latency(evaluation),
                "reliability_score": self._calculate_reliability_score(evaluation),
                "resource_efficiency": self._assess_resource_usage(evaluation),
                "scalability_potential": self._evaluate_scalability(evaluation),
                "recommended_deployment": self._suggest_deployment_strategy(evaluation)
            }
            
            evaluation["trading_assessment"] = trading_assessment
            return evaluation
        except Exception as e:
            print(f"Error evaluating model for strategy: {e}")
            return {}
    
    def _assess_trading_latency(self, evaluation: Dict) -> Dict:
        """Assess if model latency is suitable for trading"""
        inference_speed = evaluation.get("inference_speed", {})
        time_per_sample = inference_speed.get("estimated_time_per_sample_ms", float("inf"))
        
        return {
            "suitable_for_realtime": time_per_sample < 10,  # 10ms threshold
            "suitable_for_eod": time_per_sample < 1000,    # 1s threshold
            "recommended_frequency": self._get_recommended_frequency(time_per_sample),
            "optimization_needed": time_per_sample > 100    # Need optimization if >100ms
        }
    
    def _calculate_reliability_score(self, evaluation: Dict) -> float:
        """Calculate model reliability score (0-1)"""
        try:
            complexity = evaluation.get("model_complexity", {})
            memory = evaluation.get("memory_requirements", {})
            
            # Factors affecting reliability
            params_score = min(1.0, 1e8 / complexity.get("parameter_count", 1e8))
            depth_score = min(1.0, 12 / complexity.get("layer_depth", 12))
            memory_score = min(1.0, 32 / float(memory.get("minimum_ram", "32").rstrip("GB")))
            
            # Weighted average
            return (params_score * 0.4 + depth_score * 0.3 + memory_score * 0.3)
        except Exception:
            return 0.0
    
    def _assess_resource_usage(self, evaluation: Dict) -> Dict:
        """Assess resource usage efficiency"""
        memory = evaluation.get("memory_requirements", {})
        optimization = evaluation.get("optimization_potential", {})
        
        return {
            "gpu_memory_efficient": self._is_gpu_efficient(memory),
            "cpu_memory_efficient": self._is_cpu_efficient(memory),
            "batch_processing_optimal": self._is_batch_optimal(evaluation),
            "optimization_suggestions": optimization.get("memory_optimization", [])
        }
    
    def _evaluate_scalability(self, evaluation: Dict) -> Dict:
        """Evaluate model scalability potential"""
        return {
            "horizontal_scaling": self._assess_horizontal_scaling(evaluation),
            "vertical_scaling": self._assess_vertical_scaling(evaluation),
            "distributed_training": self._assess_distributed_potential(evaluation),
            "multi_gpu_support": self._assess_multi_gpu_support(evaluation)
        }
    
    def _suggest_deployment_strategy(self, evaluation: Dict) -> Dict:
        """Suggest optimal deployment strategy"""
        return {
            "recommended_infrastructure": self._get_recommended_infrastructure(evaluation),
            "scaling_strategy": self._get_scaling_strategy(evaluation),
            "backup_options": self._get_backup_options(evaluation),
            "monitoring_requirements": self._get_monitoring_requirements(evaluation)
        }
    
    def _get_recommended_frequency(self, time_per_sample: float) -> str:
        """Get recommended frequency based on latency"""
        if time_per_sample < 10:
            return "realtime"
        elif time_per_sample < 1000:
            return "eod"
        else:
            return "daily"
    
    def _is_gpu_efficient(self, memory: Dict) -> bool:
        """Check if GPU memory usage is efficient"""
        return memory.get("gpu_memory_mb", 0) < 1024
    
    def _is_cpu_efficient(self, memory: Dict) -> bool:
        """Check if CPU memory usage is efficient"""
        return memory.get("cpu_memory_mb", 0) < 2048
    
    def _is_batch_optimal(self, evaluation: Dict) -> bool:
        """Check if batch processing is optimal"""
        return evaluation.get("batch_size", 0) > 32
    
    def _assess_horizontal_scaling(self, evaluation: Dict) -> bool:
        """Assess horizontal scaling potential"""
        return evaluation.get("horizontal_scaling", False)
    
    def _assess_vertical_scaling(self, evaluation: Dict) -> bool:
        """Assess vertical scaling potential"""
        return evaluation.get("vertical_scaling", False)
    
    def _assess_distributed_potential(self, evaluation: Dict) -> bool:
        """Assess distributed training potential"""
        return evaluation.get("distributed_training", False)
    
    def _assess_multi_gpu_support(self, evaluation: Dict) -> bool:
        """Assess multi-GPU support"""
        return evaluation.get("multi_gpu_support", False)
    
    def _get_recommended_infrastructure(self, evaluation: Dict) -> str:
        """Get recommended infrastructure"""
        if evaluation.get("gpu_memory_mb", 0) > 1024:
            return "GPU"
        else:
            return "CPU"
    
    def _get_scaling_strategy(self, evaluation: Dict) -> str:
        """Get scaling strategy"""
        if evaluation.get("horizontal_scaling", False):
            return "horizontal"
        else:
            return "vertical"
    
    def _get_backup_options(self, evaluation: Dict) -> List[str]:
        """Get backup options"""
        return evaluation.get("backup_options", [])
    
    def _get_monitoring_requirements(self, evaluation: Dict) -> List[str]:
        """Get monitoring requirements"""
        return evaluation.get("monitoring_requirements", [])
    
    async def get_training_datasets(
        self,
        task: str,
        language: str = "en",
        domain: str = "finance"
    ) -> List[Dict]:
        """Get relevant training datasets"""
        try:
            return await self.hf.search_datasets(
                task=task,
                language=language,
                domain=domain
            )
        except Exception as e:
            print(f"Error fetching datasets: {e}")
            return []

    def _store_error_context(self, error_context: Dict):
        # Implement error context storage logic here
        pass
