"""
Hugging Face Integration Module
Provides secure access to HF models, datasets, and code templates.
"""
import re
import json
import ast
import asyncio
from datetime import datetime, timedelta
from pathlib import Path
from typing import Dict, List, Optional, Tuple, Set, Any
import aiohttp
import hmac
import hashlib
from huggingface_hub import (
    HfApi, 
    list_models,
    list_datasets,
    hf_hub_download,
    snapshot_download,
    InferenceClient
)
from huggingface_hub.utils import RepositoryNotFoundError
import torch

from .security.research_vault import ResearchVault

class HFIntegration:
    QUANT_TASKS = {
        "time_series": {
            "tags": ["time-series-forecasting", "financial-forecasting"],
            "metrics": ["mse", "mae", "rmse"],
            "datasets": ["yahoo_finance", "alpha_vantage"]
        },
        "market_sentiment": {
            "tags": ["sentiment-analysis", "financial-sentiment"],
            "metrics": ["accuracy", "f1"],
            "datasets": ["financial_phrasebank", "twitter_financial"]
        },
        "portfolio_optimization": {
            "tags": ["reinforcement-learning", "portfolio-management"],
            "metrics": ["sharpe_ratio", "max_drawdown"],
            "datasets": ["market_data", "portfolio_returns"]
        },
        "risk_assessment": {
            "tags": ["classification", "risk-analysis"],
            "metrics": ["auc", "precision", "recall"],
            "datasets": ["credit_risk", "market_risk"]
        },
        "anomaly_detection": {
            "tags": ["anomaly-detection", "fraud-detection"],
            "metrics": ["precision", "recall", "f1"],
            "datasets": ["market_anomalies", "fraud_data"]
        }
    }

    def __init__(self, vault: ResearchVault):
        self.vault = vault
        self.api = HfApi(token=self.vault.get_api_key("huggingface"))
        self.base_cache_dir = Path(__file__).parent / "hf_cache"
        self.model_cache = self.base_cache_dir / "models"
        self.dataset_cache = self.base_cache_dir / "datasets"
        self.template_cache = self.base_cache_dir / "templates"
        self.eval_cache = self.base_cache_dir / "evaluations"
        self.translation_model = self.vault.get_api_key("translation_model")
        self.inference_client = InferenceClient(token=self.vault.get_api_key("huggingface"))
        
        # Create cache directories
        for dir in [self.model_cache, self.dataset_cache, self.template_cache, self.eval_cache]:
            dir.mkdir(parents=True, exist_ok=True)
        
        # Initialize security headers
        self._init_security()
    
    def _init_security(self):
        """Initialize security headers per MEMORY[user_global]"""
        self.headers = {
            "Authorization": f"Bearer {self.vault.get_api_key('huggingface')}",
            "User-Agent": "RoseResearch/1.0"
        }
        
    async def search_models(
        self,
        task: Optional[str] = None,
        framework: str = "pytorch",
        dataset: Optional[str] = None,
        tags: Optional[List[str]] = None
    ) -> List[Dict]:
        """
        Search HF models with security validation and caching
        Args:
            task: Model task (e.g., 'text-classification')
            framework: Model framework (pytorch, tensorflow)
            dataset: Training dataset filter
            tags: Additional tags to filter by
        """
        cache_key = f"models_{task}_{framework}_{dataset}_{'-'.join(tags or [])}"
        cached = self._get_cached_results(self.model_cache / f"{cache_key}.json")
        if cached:
            return cached
        
        filter = {"task": task, "library": framework, "trained_dataset": dataset, "tags": tags}
        
        try:
            models = list_models(filter=filter)
            results = [await self._parse_model(m) for m in models]
            self._cache_results(self.model_cache / f"{cache_key}.json", results)
            return results
        except Exception as e:
            print(f"Error searching models: {e}")
            return []
    
    async def search_datasets(
        self,
        task: Optional[str] = None,
        language: Optional[str] = None,
        domain: Optional[str] = None
    ) -> List[Dict]:
        """Search HF datasets with domain filtering"""
        cache_key = f"datasets_{task}_{language}_{domain}"
        cached = self._get_cached_results(self.dataset_cache / f"{cache_key}.json")
        if cached:
            return cached
        
        filter = {"task": task, "language": language}
        
        try:
            datasets = list_datasets(filter=filter)
            results = [await self._parse_dataset(d) for d in datasets]
            if domain:
                results = [r for r in results if domain.lower() in r.get('description', '').lower()]
            
            self._cache_results(self.dataset_cache / f"{cache_key}.json", results)
            return results
        except Exception as e:
            print(f"Error searching datasets: {e}")
            return []
    
    async def generate_code_template(
        self,
        model_id: str,
        task: str,
        framework: str = "pytorch"
    ) -> Dict[str, str]:
        """Generate code templates from model examples"""
        cache_key = f"template_{model_id}_{task}_{framework}"
        cached = self._get_cached_results(self.template_cache / f"{cache_key}.json")
        if cached:
            return cached
        
        try:
            # Get model card and example code
            model_info = self.api.model_info(model_id)
            example_files = await self._get_example_files(model_id)
            
            # Generate template
            template = {
                "imports": self._extract_imports(example_files),
                "model_init": self._generate_model_init(model_id, task),
                "inference": self._generate_inference_code(model_id, task),
                "training": self._generate_training_template(model_id, task),
                "github_links": await self._extract_github_links(model_info)
            }
            
            self._cache_results(self.template_cache / f"{cache_key}.json", template)
            return template
        except Exception as e:
            print(f"Error generating template: {e}")
            return {}
    
    async def analyze_model_architecture(self, model_id: str) -> Dict:
        """Analyze model architecture for potential improvements"""
        try:
            # Download model config
            config_path = hf_hub_download(
                repo_id=model_id,
                filename="config.json",
                token=self.vault.get_api_key("huggingface")
            )
            
            with open(config_path) as f:
                config = json.load(f)
            
            # Analyze architecture
            analysis = {
                "architecture_type": config.get("architectures", ["Unknown"])[0],
                "parameter_count": self._calculate_params(config),
                "attention_heads": config.get("num_attention_heads"),
                "hidden_layers": config.get("num_hidden_layers"),
                "optimization_suggestions": self._generate_optimization_suggestions(config)
            }
            
            return analysis
        except Exception as e:
            print(f"Error analyzing model: {e}")
            return {}
    
    async def _parse_model(self, model) -> Dict:
        """Extract and validate model information"""
        try:
            card = await self._get_model_card(model.modelId)
            return {
                "id": model.modelId,
                "name": model.modelId.split("/")[-1],
                "task": model.pipeline_tag,
                "framework": model.library_name,
                "downloads": model.downloads,
                "likes": model.likes,
                "github_links": await self._extract_github_links(model),
                "paper_links": self._extract_paper_links(card),
                "metrics": model.metrics,
                "last_modified": model.lastModified,
                "tags": model.tags
            }
        except Exception as e:
            print(f"Error parsing model {model.modelId}: {e}")
            return {}
    
    async def _parse_dataset(self, dataset) -> Dict:
        """Parse dataset information"""
        try:
            return {
                "id": dataset.id,
                "name": dataset.id.split("/")[-1],
                "description": dataset.description,
                "citations": dataset.citation,
                "downloads": dataset.downloads,
                "likes": dataset.likes,
                "last_modified": dataset.lastModified,
                "tags": dataset.tags,
                "size": dataset.size_categories
            }
        except Exception as e:
            print(f"Error parsing dataset {dataset.id}: {e}")
            return {}
    
    async def _get_example_files(self, model_id: str) -> List[str]:
        """Get example code files from model repo"""
        try:
            files = self.api.list_repo_files(model_id)
            return [f for f in files if f.endswith('.py') and 'example' in f.lower()]
        except Exception:
            return []
    
    def _extract_imports(self, example_files: List[str]) -> List[str]:
        """Extract and normalize import statements"""
        imports = set()
        for file in example_files:
            try:
                content = self.api.hf_hub_download(file)
                tree = ast.parse(content)
                for node in ast.walk(tree):
                    if isinstance(node, (ast.Import, ast.ImportFrom)):
                        imports.add(ast.unparse(node))
            except Exception:
                continue
        return sorted(list(imports))
    
    def _generate_model_init(self, model_id: str, task: str) -> str:
        """Generate model initialization code"""
        return f"""
from transformers import AutoModelFor{task.title()}, AutoTokenizer

def init_model(model_id="{model_id}"):
    tokenizer = AutoTokenizer.from_pretrained(model_id)
    model = AutoModelFor{task.title()}.from_pretrained(model_id)
    return model, tokenizer
"""
    
    def _generate_inference_code(self, model_id: str, task: str) -> str:
        """Generate inference code template"""
        return f"""
import torch

def predict(model, tokenizer, text, max_length=512):
    inputs = tokenizer(text, return_tensors="pt", max_length=max_length, truncation=True)
    with torch.no_grad():
        outputs = model(**inputs)
    return outputs
"""
    
    def _generate_training_template(self, model_id: str, task: str) -> str:
        """Generate training code template"""
        return f"""
from transformers import Trainer, TrainingArguments

def train_model(model, tokenizer, train_dataset, eval_dataset):
    training_args = TrainingArguments(
        output_dir="./results",
        num_train_epochs=3,
        per_device_train_batch_size=8,
        per_device_eval_batch_size=8,
        warmup_steps=500,
        weight_decay=0.01,
        logging_dir="./logs",
    )
    
    trainer = Trainer(
        model=model,
        args=training_args,
        train_dataset=train_dataset,
        eval_dataset=eval_dataset
    )
    
    trainer.train()
    return trainer
"""
    
    def _calculate_params(self, config: Dict) -> int:
        """Calculate approximate parameter count"""
        try:
            hidden_size = config.get("hidden_size", 0)
            num_layers = config.get("num_hidden_layers", 0)
            vocab_size = config.get("vocab_size", 0)
            return hidden_size * num_layers * vocab_size
        except Exception:
            return 0
    
    def _generate_optimization_suggestions(self, config: Dict) -> List[str]:
        """Generate model optimization suggestions"""
        suggestions = []
        
        # Architecture suggestions
        if config.get("hidden_size", 0) > 1024:
            suggestions.append("Consider model distillation for efficiency")
        
        # Training suggestions
        if config.get("num_attention_heads", 0) > 16:
            suggestions.append("Experiment with reduced attention heads")
        
        # Hardware suggestions
        param_count = self._calculate_params(config)
        if param_count > 1e9:  # 1B params
            suggestions.append("Recommend GPU with >16GB VRAM")
        
        return suggestions
    
    def _get_cached_results(self, cache_file: Path) -> Optional[Dict]:
        """Get cached results if fresh"""
        if cache_file.exists():
            data = json.loads(cache_file.read_text())
            cached_time = datetime.fromisoformat(data["timestamp"])
            if datetime.utcnow() - cached_time < timedelta(hours=24):
                return data["results"]
        return None
    
    def _cache_results(self, cache_file: Path, results: Dict):
        """Cache results with timestamp"""
        cache_data = {
            "results": results,
            "timestamp": datetime.utcnow().isoformat()
        }
        cache_file.write_text(json.dumps(cache_data, indent=2))

    async def search_quant_models(
        self,
        task: str,
        framework: str = "pytorch",
        min_downloads: int = 1000,
        min_likes: int = 50
    ) -> List[Dict]:
        """Search for quantitative finance models"""
        if task not in self.QUANT_TASKS:
            raise ValueError(f"Task must be one of: {list(self.QUANT_TASKS.keys())}")
        
        task_config = self.QUANT_TASKS[task]
        models = await self.search_models(
            task=task,
            framework=framework,
            tags=task_config["tags"]
        )
        
        # Filter by popularity
        filtered_models = [
            m for m in models 
            if m.get("downloads", 0) >= min_downloads 
            and m.get("likes", 0) >= min_likes
        ]
        
        # Add task-specific metrics
        for model in filtered_models:
            model["task_metrics"] = task_config["metrics"]
            model["recommended_datasets"] = task_config["datasets"]
        
        return filtered_models

    async def evaluate_model_metrics(
        self,
        model_id: str,
        task: str,
        sample_data: Optional[Dict] = None
    ) -> Dict:
        """Evaluate model performance metrics"""
        if task not in self.QUANT_TASKS:
            raise ValueError(f"Task must be one of: {list(self.QUANT_TASKS.keys())}")
            
        cache_key = f"eval_{model_id}_{task}"
        cached = self._get_cached_results(self.eval_cache / f"{cache_key}.json")
        if cached:
            return cached
            
        try:
            # Get model config and task metrics
            config = await self.analyze_model_architecture(model_id)
            task_metrics = self.QUANT_TASKS[task]["metrics"]
            
            # Evaluate model characteristics
            evaluation = {
                "model_complexity": self._evaluate_complexity(config),
                "memory_requirements": self._estimate_memory_usage(config),
                "inference_speed": await self._benchmark_inference(model_id),
                "task_suitability": self._evaluate_task_fit(config, task),
                "recommended_metrics": task_metrics,
                "optimization_potential": self._evaluate_optimization_potential(config)
            }
            
            # Add sample performance if data provided
            if sample_data:
                evaluation["sample_performance"] = await self._evaluate_sample_performance(
                    model_id, task, sample_data
                )
            
            self._cache_results(self.eval_cache / f"{cache_key}.json", evaluation)
            return evaluation
            
        except Exception as e:
            print(f"Error evaluating model metrics: {e}")
            return {}

    def _evaluate_complexity(self, config: Dict) -> Dict:
        """Evaluate model architectural complexity"""
        return {
            "parameter_count": self._calculate_params(config),
            "layer_depth": config.get("num_hidden_layers", 0),
            "attention_complexity": self._calculate_attention_complexity(config),
            "memory_access_pattern": self._analyze_memory_pattern(config)
        }

    def _estimate_memory_usage(self, config: Dict) -> Dict:
        """Estimate model memory requirements"""
        params = self._calculate_params(config)
        batch_size = 32  # Default batch size
        
        return {
            "model_size_mb": (params * 4) / (1024 * 1024),  # 4 bytes per parameter
            "batch_memory_mb": (params * 4 * batch_size) / (1024 * 1024),
            "recommended_gpu_memory": f"{max(8, (params * 4 * 3) // (1024 * 1024 * 1024))}GB",
            "minimum_ram": f"{max(16, (params * 4 * 4) // (1024 * 1024 * 1024))}GB"
        }

    async def _benchmark_inference(self, model_id: str) -> Dict:
        """Benchmark model inference speed"""
        try:
            config = await self.analyze_model_architecture(model_id)
            
            # Estimate based on model architecture
            return {
                "estimated_time_per_sample_ms": self._estimate_inference_time(config),
                "batch_processing_capability": self._estimate_batch_capability(config),
                "gpu_utilization_expected": self._estimate_gpu_usage(config)
            }
        except Exception as e:
            print(f"Error benchmarking inference: {e}")
            return {}

    def _evaluate_task_fit(self, config: Dict, task: str) -> Dict:
        """Evaluate model suitability for specific task"""
        task_config = self.QUANT_TASKS[task]
        architecture = config.get("architectures", ["Unknown"])[0]
        
        return {
            "architecture_match": self._calculate_architecture_match(architecture, task),
            "feature_compatibility": self._check_feature_compatibility(config, task_config),
            "recommended_adjustments": self._suggest_adjustments(config, task),
            "transfer_learning_potential": self._evaluate_transfer_potential(config, task)
        }

    def _evaluate_optimization_potential(self, config: Dict) -> Dict:
        """Evaluate potential optimizations"""
        return {
            "quantization_suitable": self._check_quantization_suitability(config),
            "pruning_potential": self._estimate_pruning_potential(config),
            "parallelization_options": self._suggest_parallel_strategies(config),
            "memory_optimization": self._suggest_memory_optimizations(config)
        }

    async def _evaluate_sample_performance(
        self,
        model_id: str,
        task: str,
        sample_data: Dict
    ) -> Dict:
        """Evaluate model performance on sample data"""
        try:
            metrics = self.QUANT_TASKS[task]["metrics"]
            results = {}
            
            for metric in metrics:
                results[metric] = await self._calculate_metric(
                    model_id, metric, sample_data
                )
            
            return {
                "metrics": results,
                "performance_category": self._categorize_performance(results),
                "improvement_suggestions": self._suggest_improvements(results, task)
            }
        except Exception as e:
            print(f"Error evaluating sample performance: {e}")
            return {}

    async def analyze_data(self, data: Dict[str, Any]) -> Dict[str, Any]:
        """Analyze market data and generate insights"""
        try:
            # Create analysis prompt
            prompt = self._create_analysis_prompt(data)
            
            # Generate analysis using the model
            response = self.inference_client.text_generation(
                prompt,
                model=self.translation_model,
                max_new_tokens=512,  # Reduced from 1024
                temperature=0.7,
                top_p=0.95
            )
            
            # Parse and structure the response
            analysis = self._parse_analysis(response)
            return analysis
            
        except Exception as e:
            print(f"Error analyzing data: {e}")
            return {
                "error": str(e),
                "status": "failed",
                "timestamp": datetime.utcnow().isoformat()
            }
            
    def _create_analysis_prompt(self, data: Dict[str, Any]) -> str:
        """Create a detailed prompt for data analysis"""
        prompt = "Analyze the following market data and provide key insights:\n\n"
        
        # Add data points to prompt
        if isinstance(data, dict) and "data" in data:
            for source, source_data in data["data"].items():
                if isinstance(source_data, dict):
                    prompt += f"\n{source} Data:\n"
                    prompt += self._format_data_points(source_data)
                
        prompt += "\nProvide analysis including:\n"
        prompt += "1. Key trends and patterns\n"
        prompt += "2. Market indicators and their implications\n"
        prompt += "3. Notable changes from previous periods\n"
        prompt += "4. Potential factors influencing the trends\n"
        prompt += "5. Future outlook and recommendations\n"
        
        return prompt
        
    def _format_data_points(self, data: Dict[str, Any], indent: str = "  ") -> str:
        """Format data points for the prompt"""
        result = ""
        if isinstance(data, dict):
            for key, value in data.items():
                if isinstance(value, dict):
                    result += f"{indent}- {key}:\n"
                    result += self._format_data_points(value, indent + "  ")
                else:
                    result += f"{indent}- {key}: {value}\n"
        return result
        
    def _parse_analysis(self, response: str) -> Dict[str, Any]:
        """Parse and structure the analysis response"""
        return {
            "raw_analysis": response,
            "status": "success",
            "timestamp": datetime.utcnow().isoformat()
        }
        
    async def generate_section(self, section: str, analysis: Dict[str, Any]) -> str:
        """Generate a section of the research paper"""
        try:
            # Create a detailed prompt for the section
            prompt = self._create_section_prompt(section, analysis)
            
            # Generate content using the model
            response = self.inference_client.text_generation(
                prompt,
                model=self.translation_model,
                max_new_tokens=512,  # Reduced from 1024
                temperature=0.7,
                top_p=0.95
            )
            
            return response
            
        except Exception as e:
            print(f"Error generating section {section}: {e}")
            return f"Error generating {section} section. Please try again."
            
    def _create_section_prompt(self, section: str, analysis: Dict[str, Any]) -> str:
        """Create a detailed prompt for the section"""
        base_prompt = f"Write a detailed {section} section for a research paper on {analysis.get('topic', '')}.\n\n"
        
        if section == "Executive Summary":
            return base_prompt + """
            Include:
            1. Overview of key findings
            2. Market performance summary
            3. Critical trends
            4. Strategic recommendations
            """
            
        elif section == "Introduction":
            return base_prompt + """
            Include:
            1. Context and background
            2. Significance of the research
            3. Research objectives
            4. Brief overview of methodology
            """
            
        elif section == "Methodology":
            return base_prompt + """
            Include:
            1. Data collection methods
            2. Analysis techniques
            3. Tools and frameworks used
            4. Limitations and assumptions
            """
            
        elif section == "Results":
            data_points = self._format_data_points(analysis.get("data", {}))
            return base_prompt + f"""
            Analyze and present the following data points:
            {data_points}
            
            Include:
            1. Key findings
            2. Statistical analysis
            3. Trends and patterns
            4. Visual representation suggestions
            """
            
        elif section == "Discussion":
            return base_prompt + """
            Include:
            1. Interpretation of results
            2. Comparison with existing literature
            3. Implications for industry
            4. Future research directions
            """
            
        elif section == "Conclusion":
            return base_prompt + """
            Include:
            1. Summary of key findings
            2. Research contributions
            3. Practical implications
            4. Final recommendations
            """
            
        return base_prompt
