"""
Research Agent Module
Implements research paper generation with multi-source data integration.
"""
import json
from datetime import datetime
from typing import Dict, Any, List, Optional
from pathlib import Path
from aiohttp import web

from .core_integration import ResearchCore
from .hf_integration import HFIntegration
from SYS_RESEARCH.security.research_vault import ResearchVault
from SYS_RESEARCH.web.app import init_app

class RoseResearch:
    def __init__(self):
        self.core = ResearchCore()
        self.hf_service = HFIntegration(self.core.vault)
        self.vault = ResearchVault()
        
    async def get_search_progress(self):
        return self.core.get_progress()
        
    async def validate_request(self, request):
        auth_header = request.headers.get('Authorization')
        role_header = request.headers.get('X-System-Role')
        
        # Validate required security headers
        if not auth_header or not role_header:
            raise web.HTTPUnauthorized(text='Missing security headers')
            
        # Verify HMAC signature format
        if not auth_header.startswith('HMAC-SHA256 '):
            raise web.HTTPForbidden(text='Invalid auth scheme')
            
        token = auth_header.split(' ')[1]
        try:
            payload = await self.vault.validate_token(token)
            
            # Verify role-based access
            if payload.get('role') != role_header:
                raise web.HTTPForbidden(text='Role mismatch')
                
            request['user'] = payload
            
        except web.HTTPError:
            raise
        except Exception as e:
            self.logger.error(f'Auth failure: {str(e)}')
            raise web.HTTPInternalServerError()
            
    async def generate_research_paper(self, request):
        await self.validate_request(request)
        
        if 'research:write' not in request['user'].get('scopes', []):
            raise web.HTTPForbidden(text='Missing required scope')
        
        topic = request.query.get("topic")
        time_period = request.query.get("time_period")
        """Generate a research paper on the given topic"""
        print(f"Generating research paper on {topic}...")
        
        # Gather market data
        raw_data = await self._gather_market_data(topic, time_period)
        
        # Analyze data
        analysis = await self._analyze_data(raw_data)
        
        # Generate paper sections
        paper = await self._generate_paper(topic, analysis)
        
        # Save output
        output = self._save_output(paper, topic, time_period)
        
        return output
        
    async def _gather_market_data(self, topic: str, time_period: str) -> Dict[str, Any]:
        """Gather market data from various sources"""
        data = {}
        
        # List of data sources to query
        sources = ["census.gov", "fred.stlouisfed.org", "bloomberg.com", "statista.com"]
        
        for source in sources:
            try:
                source_data = await self.core.fetch_data(source, {"topic": topic, "period": time_period})
                if source_data:
                    data[source] = source_data
            except Exception as e:
                print(f"Warning: Failed to fetch data from {source}: {e}")
                
        return {
            "topic": topic,
            "time_period": time_period,
            "data": data
        }
        
    async def _analyze_data(self, raw_data: Dict[str, Any]) -> Dict[str, Any]:
        """Analyze gathered market data"""
        try:
            # Analyze each data source
            analysis = await self.hf_service.analyze_data(raw_data)
            
            return {
                "topic": raw_data["topic"],
                "time_period": raw_data["time_period"],
                "data": raw_data["data"],
                "analysis": analysis
            }
            
        except Exception as e:
            print(f"Error analyzing data: {e}")
            return raw_data
            
    async def _generate_paper(self, topic: str, analysis: Dict[str, Any]) -> Dict[str, Any]:
        """Generate the research paper content"""
        paper = {
            "title": f"Market Analysis: {topic}",
            "timestamp": datetime.utcnow().isoformat(),
            "sections": [
                "Executive Summary",
                "Market Overview",
                "Data Analysis",
                "Key Findings",
                "Future Outlook",
                "Methodology"
            ],
            "content": {},
            "visualizations": []
        }
        
        # Generate each section
        for section in paper["sections"]:
            try:
                content = await self.hf_service.generate_section(section, analysis)
                paper["content"][section] = content
            except Exception as e:
                print(f"Error generating section {section}: {e}")
                paper["content"][section] = f"Error generating {section} section. Please try again."
                
        return paper
        
    def _save_output(self, paper: Dict[str, Any], topic: str, time_period: str) -> Dict[str, Any]:
        """Save the generated paper to a file"""
        try:
            # Add security headers
            paper["security"] = {
                "system_role": "Research",
                "task_signature": self._generate_signature(paper),
                "knowledge_graph_id": f"ROO-ROSEREF-{datetime.utcnow().isoformat()}"
            }
            
            # Create output directory if it doesn't exist
            output_dir = Path("research_output")
            output_dir.mkdir(exist_ok=True)
            
            # Generate filename
            timestamp = datetime.utcnow().strftime("%Y%m%d_%H%M%S")
            filename = f"{topic.lower().replace(' ', '_')}_{time_period.lower()}_{timestamp}.json"
            
            # Save to file
            output_path = output_dir / filename
            with open(output_path, "w") as f:
                json.dump(paper, f, indent=2)
                
            print(f"Paper generated successfully: {output_path}")
            return paper
            
        except Exception as e:
            error_context = {
                "timestamp": datetime.utcnow().isoformat(),
                "error_type": type(e).__name__,
                "error_message": str(e),
                "stack_trace": e.__traceback__
            }
            
            # Log error for debugging
            print(f"Error in research agent: {error_context}")
            raise
            
    def _generate_signature(self, paper: Dict[str, Any]) -> str:
        """Generate a signature for the paper"""
        try:
            return self.core.vault.sign_request({
                "title": paper["title"],
                "timestamp": paper["timestamp"],
                "sections": paper["sections"]
            })
        except Exception as e:
            print(f"Error generating signature: {e}")
            return ""
