from pydantic import BaseModel
from redis import Redis
from datetime import datetime, timezone, timedelta
import secrets

class DeviceCode(BaseModel):
    device_code: str
    user_code: str
    client_id: str
    status: str = 'pending'
    expires_in: int
    created_at: datetime = datetime.now(timezone.utc)
    expires_at: datetime = None
    
    @classmethod
    def generate(cls, client_id: str) -> 'DeviceCode':
        code = cls(
            device_code=secrets.token_urlsafe(32),
            user_code='-'.join([secrets.token_hex(2) for _ in range(4)]),
            client_id=client_id,
            expires_in=600,
            expires_at=datetime.now(timezone.utc) + timedelta(minutes=10)
        )
        return code

class OAuthService:
    def __init__(self, redis: Redis):
        self.redis = redis
        
    async def create_device_code(self, client_id: str) -> DeviceCode:
        code = DeviceCode.generate(client_id)
        await self.redis.set(
            f'oauth:device:{code.device_code}',
            code.json(),
            ex=code.expires_in
        )
        return code

    async def verify_device_code(self, device_code: str, user_code: str) -> bool:
        data = await self.redis.get(f'oauth:device:{device_code}')
        if not data:
            return False
            
        code = DeviceCode.parse_raw(data)
        if code.user_code != user_code:
            return False
            
        code.status = 'approved'
        await self.redis.set(
            f'oauth:device:{device_code}',
            code.json(),
            ex=code.expires_in
        )
        return True
