import json
from datetime import datetime, timezone
from typing import Dict, Optional
import aiohttp
import logging
import hmac

logger = logging.getLogger(__name__)

class PhaseTracker:
    def __init__(self, vault, secret_key):
        self.vault = vault
        self.secret_key = secret_key.encode()
        self.phases = {}
        self.current_phase = "INITIALIZING"
        self.phase_requirements = {
            "PHASE3": {
                "websocket": True,
                "crdt": True,
                "presence": True,
                "encryption": "FIPS-140-3"
            }
        }
    
    async def set_phase(self, phase: str) -> bool:
        phase_data = {
            "phase": phase,
            "timestamp": datetime.now(timezone.utc).isoformat(),
            "requirements_met": await self._verify_requirements(phase)
        }
        
        signature = self.start_phase(phase)
        phase_data["signature"] = signature
        
        await self.vault.redis.set(
            "system:phase",
            json.dumps(phase_data)
        )
        self.current_phase = phase
        return True
        
    async def get_status(self) -> Dict:
        phase_data = await self.vault.redis.get("system:phase")
        if not phase_data:
            return {
                "phase": "UNKNOWN",
                "timestamp": datetime.now(timezone.utc).isoformat(),
                "status": "ERROR"
            }
        return json.loads(phase_data)
    
    async def _verify_requirements(self, phase: str) -> bool:
        if phase != "PHASE3":
            return True
            
        try:
            # Check WebSocket availability
            ws_status = await self._check_websocket()
            if not ws_status:
                return False
                
            # Verify CRDT operations
            crdt_status = await self._check_crdt()
            if not crdt_status:
                return False
                
            # Test presence tracking
            presence_status = await self._check_presence()
            if not presence_status:
                return False
                
            return True
        except Exception as e:
            logger.error(f"Phase 3 verification failed: {e}")
            return False
            
    async def _check_websocket(self) -> bool:
        try:
            # Verify WebSocket endpoint is responding
            async with aiohttp.ClientSession() as session:
                async with session.ws_connect(
                    "ws://localhost:8000/collab/ws",
                    headers={"X-System-Role": "Research"}
                ) as ws:
                    return True
        except Exception:
            return False
            
    async def _check_crdt(self) -> bool:
        # Verify CRDT operations are working
        return await self.vault.redis.ping()
        
    async def _check_presence(self) -> bool:
        # Verify presence tracking is active
        key = "presence:test"
        await self.vault.redis.set(key, "test", ex=60)
        return await self.vault.redis.get(key) is not None

    def start_phase(self, phase_name):
        timestamp = datetime.utcnow().isoformat()
        signature = hmac.new(
            self.secret_key,
            f"{phase_name}|{timestamp}".encode(),
            'sha256'
        ).hexdigest()
        self.phases[phase_name] = {
            'start': timestamp,
            'signature': signature
        }
        return signature

    def validate_phase(self, phase_name, signature):
        phase = self.phases.get(phase_name)
        if not phase:
            return False
        expected = hmac.new(
            self.secret_key,
            f"{phase_name}|{phase['start']}".encode(),
            'sha256'
        ).hexdigest()
        return hmac.compare_digest(expected, signature)
