"""
Research Vault for secure credential management.
Implements MEMORY[user_global] security protocols with HMAC signing and key rotation.
"""
import os
import json
import hmac
import hashlib
import secrets
from datetime import datetime, timedelta
from pathlib import Path
from cryptography.fernet import Fernet
from typing import Optional, Dict, Any
import time
from aiohttp import web
import jwt
from .oauth_models import DeviceCode, OAuthService
from queue import Queue
from threading import Thread
import logging

logger = logging.getLogger(__name__)

class ResearchVault:
    def __init__(self):
        self.vault_path = Path(__file__).parent / ".vault"
        self.vault_path.mkdir(exist_ok=True)
        
        # Initialize encryption
        self._init_encryption()
        
        # Load or create vault
        self.vault = self._load_vault()
        
        # Initialize key rotation
        self.keys = {}
        self.key_versions = {}
        
        # Initialize OAuth service
        self.oauth = OAuthService(self.redis)
        
    def _init_encryption(self):
        """Initialize encryption key"""
        key_file = self.vault_path / "key.bin"
        if not key_file.exists():
            key = Fernet.generate_key()
            with open(key_file, "wb") as f:
                f.write(key)
        else:
            with open(key_file, "rb") as f:
                key = f.read()
                
        self.fernet = Fernet(key)
        
    def _load_vault(self) -> Dict:
        """Load vault contents"""
        vault_file = self.vault_path / "vault.enc"
        if not vault_file.exists():
            return {}
            
        with open(vault_file, "rb") as f:
            encrypted = f.read()
            decrypted = self.fernet.decrypt(encrypted)
            return json.loads(decrypted)
            
    def _save_vault(self):
        """Save vault contents"""
        vault_file = self.vault_path / "vault.enc"
        encrypted = self.fernet.encrypt(json.dumps(self.vault).encode())
        
        with open(vault_file, "wb") as f:
            f.write(encrypted)
            
    def get_api_key(self, service: str) -> Optional[str]:
        """Get API key for service"""
        if service not in self.vault:
            return None
            
        key_data = self.vault[service]
        if self._is_key_expired(key_data):
            self._rotate_key(service)
            
        return self.vault[service]["key"]
        
    def set_api_key(self, service: str, key: str):
        """Set API key for service"""
        self.vault[service] = {
            "key": key,
            "created": datetime.utcnow().isoformat(),
            "expires": (datetime.utcnow() + timedelta(hours=24)).isoformat()
        }
        self._save_vault()
        
    def get_oauth_credentials(self, service: str) -> Optional[Dict[str, str]]:
        """Get OAuth credentials for service"""
        if f"{service}_oauth" not in self.vault:
            return None
            
        creds = self.vault[f"{service}_oauth"]
        if self._is_key_expired(creds):
            self._rotate_oauth_credentials(service)
            
        return {
            "client_id": creds["client_id"],
            "client_secret": creds["client_secret"],
            "access_token": creds.get("access_token"),
            "refresh_token": creds.get("refresh_token")
        }
        
    def set_oauth_credentials(self, service: str, client_id: str, client_secret: str, 
                            access_token: Optional[str] = None, refresh_token: Optional[str] = None):
        """Set OAuth credentials for service"""
        self.vault[f"{service}_oauth"] = {
            "client_id": client_id,
            "client_secret": client_secret,
            "access_token": access_token,
            "refresh_token": refresh_token,
            "created": datetime.utcnow().isoformat(),
            "expires": (datetime.utcnow() + timedelta(hours=24)).isoformat()
        }
        self._save_vault()
        
    def _is_key_expired(self, key_data: Dict) -> bool:
        """Check if key is expired"""
        expires = datetime.fromisoformat(key_data["expires"])
        return datetime.utcnow() > expires
        
    def _rotate_key(self, service_name: str):
        """Rotate API key for a service"""
        current_key = self.keys.get(service_name)
        new_key = secrets.token_urlsafe(64)
        
        # Store previous version for 24h grace period
        self.key_versions[service_name] = {
            "previous": current_key,
            "current": new_key,
            "valid_until": datetime.now() + timedelta(hours=24)
        }
        
        self.keys[service_name] = new_key
        
    def _rotate_oauth_credentials(self, service: str):
        """Rotate OAuth credentials"""
        # TODO: Implement OAuth credential rotation
        pass
        
    def sign_request(self, payload: dict) -> str:
        """Sign request with HMAC"""
        knowledge_graph_id = f"ROO-ROSEREF-{int(time.time())}"
        message = f"{json.dumps(payload)}|{knowledge_graph_id}"
        return hmac.new(
            self.get_api_key('signing').encode(),
            message.encode(),
            hashlib.sha256
        ).hexdigest()
        
    def generate_headers(self, service: str) -> Dict[str, str]:
        """Generate headers for service request"""
        api_key = self.get_api_key(service)
        if not api_key:
            raise ValueError(f"API key not found for service: {service}")
            
        headers = {
            "Authorization": f"Bearer {api_key}",
            "User-Agent": "RoseResearch/1.0",
            "system_role": "Research",
            "task_signature": self.sign_request({
                "service": service,
                "timestamp": datetime.utcnow().isoformat()
            })
        }
        return headers

    async def store_key_cookie(self, response: web.Response, key_name: str):
        encrypted = self.fernet.encrypt(self.keys[key_name].encode())
        response.set_cookie(
            name=f'vault_{key_name}',
            value=encrypted.decode(),
            httponly=True,
            secure=True,
            samesite='Strict'
        )

    async def load_key_cookie(self, request: web.Request, key_name: str) -> str:
        encrypted = request.cookies.get(f'vault_{key_name}')
        return self.fernet.decrypt(encrypted.encode()).decode()

    async def generate_signed_token(self, client_id: str, scope: str) -> str:
        payload = {
            'client_id': client_id,
            'scope': scope,
            'exp': datetime.now(timezone.utc) + timedelta(hours=1)
        }
        return self.sign_request(payload)

    async def validate_token(self, token: str) -> dict:
        try:
            payload = jwt.decode(
                token,
                self.get_api_key('signing'),
                algorithms=['HS256']
            )
            return payload
        except jwt.ExpiredSignatureError:
            raise web.HTTPUnauthorized(text='Token expired')
        except jwt.InvalidTokenError:
            raise web.HTTPForbidden(text='Invalid token')

    async def rotate_refresh_token(self, old_token: str) -> str:
        try:
            payload = await self.validate_token(old_token)
            new_token = self.generate_signed_token(
                client_id=payload['client_id'],
                scope=payload['scope']
            )
            
            # Revoke old token immediately
            await self.redis.set(
                f'token:revoked:{old_token}', 
                '1', 
                ex=payload['exp'] - int(time.time())
            )
            return new_token
        except web.HTTPError:
            raise
        except Exception as e:
            self.logger.error(f'Token rotation failed: {str(e)}')
            raise web.HTTPInternalServerError()

    def verify_model_hash(self, model_state_dict, expected_hash):
        '''Verify model integrity using SHA-256 hash'''
        serialized = json.dumps(model_state_dict, sort_keys=True)
        current_hash = hashlib.sha256(serialized.encode()).hexdigest()
        
        if current_hash != expected_hash:
            self.log_security_alert(
                alert_type='MODEL_TAMPERING',
                details={
                    'expected_hash': expected_hash[-6:],
                    'actual_hash': current_hash[-6:],
                    'model_size': len(serialized)
                },
                severity=10
            )
        return current_hash == expected_hash

class HybridResearchVault(ResearchVault):
    def __init__(self):
        super().__init__()
        self.aes_key = os.getenv('VAULT_AES_KEY').encode()
        self.cipher = Fernet(self.aes_key)
        self.encryption_queue = Queue(maxsize=10)
        self.decryption_queue = Queue(maxsize=10)
        self._start_workers()

    def _start_workers(self):
        # Encryption worker
        Thread(target=self._process_encryption, daemon=True).start()
        # Decryption worker
        Thread(target=self._process_decryption, daemon=True).start()

    def _process_encryption(self):
        while True:
            item = self.encryption_queue.get()
            try:
                encrypted = self._aes_encrypt(item['data'])
                self._save_encrypted(encrypted, item['path'])
            except Exception as e:
                logger.error(f"Encryption failed: {str(e)}")
            finally:
                self.encryption_queue.task_done()

    def async_save_model(self, state_dict, path):
        """Non-blocking model save"""
        self.encryption_queue.put({
            'data': state_dict,
            'path': path
        })

    def _process_decryption(self):
        while True:
            item = self.decryption_queue.get()
            try:
                decrypted = self._aes_decrypt(item['data'])
                self.cache[item['path']] = decrypted
            except Exception as e:
                logger.error(f"Decryption failed: {str(e)}")
            finally:
                self.decryption_queue.task_done()

    def async_load_model(self, path):
        """Queue model for background decryption"""
        encrypted = self._load_encrypted(path)
        self.decryption_queue.put({
            'data': encrypted,
            'path': path
        })

    def _encrypt_state_dict(self, state_dict):
        serialized = json.dumps(state_dict)
        return self.cipher.encrypt(serialized.encode())

    def _decrypt_state_dict(self, encrypted_data):
        decrypted = self.cipher.decrypt(encrypted_data)
        return json.loads(decrypted.decode())

    def save_model(self, model, use_aes=False):
        if use_aes:
            encrypted = self._encrypt_state_dict(model.state_dict())
            self._store_encrypted(encrypted)
        else:
            super().save_model(model)

    def load_model(self, model_id, use_aes=False):
        if use_aes:
            encrypted = self._retrieve_encrypted(model_id)
            return self._decrypt_state_dict(encrypted)
        return super().load_model(model_id)
