"""
Translation Pipeline using Gemini model
Handles multi-language support with context preservation for academic papers.
"""
import re
import json
from typing import Dict, List, Optional, Tuple, Set
from pathlib import Path
import asyncio
from datetime import datetime, timedelta
import hashlib
import aiohttp
from huggingface_hub import InferenceClient

from .security.research_vault import ResearchVault

class TranslationPipeline:
    def __init__(self):
        self.vault = ResearchVault()
        self.model = self.vault.get_api_key("translation_model")
        self.client = InferenceClient(token=self.vault.get_api_key("huggingface"))
        self.cache_dir = Path(__file__).parent / "translation_cache"
        self.cache_dir.mkdir(exist_ok=True)
        
    async def translate(self, text: str, source_lang: str = "fr", target_lang: str = "en") -> str:
        """
        Translate text using Gemini model
        """
        cache_key = self._cache_key(text, source_lang, target_lang)
        cached = self._get_cached_translation(cache_key)
        if cached:
            return cached
            
        try:
            prompt = f"Translate the following text from {source_lang} to {target_lang}:\n\n{text}"
            
            response = self.client.text_generation(
                prompt,
                model=self.model,
                max_new_tokens=512,
                temperature=0.3
            )
            
            self._cache_translation(cache_key, response)
            return response
            
        except Exception as e:
            print(f"Translation error: {e}")
            return text  # Return original text if translation fails
            
    def _cache_key(self, text: str, source_lang: str, target_lang: str) -> str:
        """Generate cache key for translation"""
        content = f"{source_lang}_{target_lang}_{hash(text)}"
        return hashlib.sha256(content.encode()).hexdigest()
        
    def _get_cached_translation(self, cache_key: str) -> Optional[str]:
        """Get cached translation if available"""
        cache_file = self.cache_dir / f"{cache_key}.json"
        if not cache_file.exists():
            return None
            
        with open(cache_file) as f:
            data = json.load(f)
            if datetime.fromisoformat(data["expires"]) > datetime.utcnow():
                return data["translation"]
        return None
        
    def _cache_translation(self, cache_key: str, translation: str):
        """Cache translation result"""
        cache_file = self.cache_dir / f"{cache_key}.json"
        data = {
            "translation": translation,
            "created": datetime.utcnow().isoformat(),
            "expires": (datetime.utcnow() + timedelta(days=7)).isoformat()
        }
        with open(cache_file, "w") as f:
            json.dump(data, f)
