import httpx
import asyncio
from typing import Dict, List, Optional
from datetime import datetime

class WebSearchAgent:
    """Agent for performing web searches and data collection"""
    
    def __init__(self):
        self.client = httpx.AsyncClient(timeout=30.0)
        self.sources = {
            "financial": [
                "bloomberg.com",
                "reuters.com",
                "ft.com"
            ],
            "retail": [
                "census.gov/retail",
                "nrf.com",
                "retaildive.com"
            ],
            "economic": [
                "fred.stlouisfed.org",
                "worldbank.org",
                "imf.org"
            ]
        }
        
    async def search(self, query: str, category: str = None) -> List[Dict]:
        """Perform a web search for the given query"""
        try:
            sources = self.sources.get(category, []) if category else []
            results = []
            
            # Perform searches
            tasks = [
                self._search_source(query, source)
                for source in sources
            ]
            
            # Gather results
            search_results = await asyncio.gather(*tasks, return_exceptions=True)
            
            # Process results
            for result in search_results:
                if isinstance(result, Exception):
                    continue
                results.extend(result)
                
            return self._validate_results(results)
            
        except Exception as e:
            print(f"Search error: {e}")
            return []
            
    async def _search_source(self, query: str, source: str) -> List[Dict]:
        """Search a specific source"""
        try:
            # Add source-specific parameters
            params = {
                "q": query,
                "source": source,
                "timestamp": datetime.utcnow().isoformat()
            }
            
            # Make request
            response = await self.client.get(
                f"https://{source}/api/search",
                params=params
            )
            
            return response.json()
            
        except Exception as e:
            print(f"Error searching {source}: {e}")
            return []
            
    def _validate_results(self, results: List[Dict]) -> List[Dict]:
        """Validate and filter search results"""
        validated = []
        
        for result in results:
            if self._is_valid_result(result):
                validated.append(result)
                
        return validated
        
    def _is_valid_result(self, result: Dict) -> bool:
        """Check if a search result is valid"""
        required_fields = ["title", "url", "source", "timestamp"]
        return all(field in result for field in required_fields)
        
    async def close(self):
        """Close the HTTP client"""
        await self.client.aclose()
