"""
Research Vault for secure credential management.
Implements MEMORY[user_global] security protocols with HMAC signing and key rotation.
"""
import os
import json
import hmac
import hashlib
from datetime import datetime, timedelta
from pathlib import Path
from cryptography.fernet import Fernet
from typing import Optional, Dict, Any

class ResearchVault:
    def __init__(self):
        self.vault_path = Path(__file__).parent / ".vault"
        self.vault_path.mkdir(exist_ok=True)
        
        # Initialize encryption
        self._init_encryption()
        
        # Set up initial credentials
        self.credentials = {
            "CORE": {
                "api_key": "xwbWrtZ4fKk5NcVREDaJ10AmHlUiLu9j",
                "rotation": "daily",
                "last_rotated": datetime.utcnow().isoformat()
            },
            "SCIENCEOPEN": {
                "oauth": {
                    "client_id": self._encrypt_value(os.getenv("SCIENCEOPEN_CLIENT_ID", "")),
                    "client_secret": self._encrypt_value(os.getenv("SCIENCEOPEN_CLIENT_SECRET", "")),
                },
                "rotation": "weekly"
            },
            "HAL": {
                "orcid": self._encrypt_value(os.getenv("HAL_ORCID", "")),
                "rotation": "monthly"
            },
            "DEEPL": {
                "api_key": self._encrypt_value(os.getenv("DEEPL_API_KEY", "")),
                "rotation": "monthly"
            }
        }
        
        # Save initial state
        self._save_vault()
    
    def _init_encryption(self):
        """Initialize Fernet encryption"""
        key_file = self.vault_path / "vault.key"
        if not key_file.exists():
            self.key = Fernet.generate_key()
            key_file.write_bytes(self.key)
        else:
            self.key = key_file.read_bytes()
        self.cipher = Fernet(self.key)
    
    def _encrypt_value(self, value: str) -> bytes:
        """Encrypt a value using Fernet"""
        return self.cipher.encrypt(value.encode())
    
    def _decrypt_value(self, value: bytes) -> str:
        """Decrypt a value using Fernet"""
        return self.cipher.decrypt(value).decode()
    
    def _save_vault(self):
        """Save vault state securely"""
        vault_file = self.vault_path / "vault.json"
        with vault_file.open('w') as f:
            json.dump(self.credentials, f, indent=2)
    
    def get_credential(self, service: str, key: str) -> str:
        """Get a credential with automatic rotation check"""
        if service not in self.credentials:
            raise KeyError(f"Service {service} not found in vault")
            
        cred = self.credentials[service]
        
        # Check if rotation needed
        if self._needs_rotation(service):
            self.rotate_credentials(service)
        
        # Return requested credential
        if isinstance(cred.get(key), bytes):
            return self._decrypt_value(cred[key])
        return cred.get(key, "")
    
    def _needs_rotation(self, service: str) -> bool:
        """Check if credentials need rotation"""
        cred = self.credentials[service]
        last_rotated = datetime.fromisoformat(cred.get('last_rotated', '2000-01-01'))
        
        rotation_schedule = {
            "daily": timedelta(days=1),
            "weekly": timedelta(weeks=1),
            "monthly": timedelta(days=30)
        }
        
        return datetime.utcnow() - last_rotated > rotation_schedule[cred['rotation']]
    
    def rotate_credentials(self, service: str):
        """Implement MCP_VAULT_ROTATE_24H for credentials"""
        cred = self.credentials[service]
        
        if service == "CORE":
            # For CORE, we rotate the signature key
            signature_key = Fernet.generate_key()
            cred['signature_key'] = self._encrypt_value(signature_key.decode())
        
        cred['last_rotated'] = datetime.utcnow().isoformat()
        self._save_vault()
    
    def generate_headers(self, service: str) -> Dict[str, str]:
        """Generate secure headers per MEMORY[user_global]"""
        timestamp = datetime.utcnow().isoformat()
        
        if service == "CORE":
            api_key = self.get_credential("CORE", "api_key")
            message = f"{service}|{timestamp}"
            signature = hmac.new(
                key=api_key.encode(),
                msg=message.encode(),
                digestmod=hashlib.sha256
            ).hexdigest()
            
            return {
                "Authorization": f"Bearer {api_key}",
                "X-Signature": f"{timestamp}:{signature}",
                "X-KG-ID": f"ROO-ROSEREF-{int(datetime.utcnow().timestamp())}"
            }
        
        return {}
    
    def get_oauth_credentials(self, service: str) -> Dict[str, str]:
        """Get OAuth credentials for services using ORCID"""
        if service in ["SCIENCEOPEN", "HAL"]:
            return {
                "client_id": self.get_credential(service, "oauth")["client_id"],
                "client_secret": self.get_credential(service, "oauth")["client_secret"]
            }
        return {}
