from datetime import datetime
from typing import Dict, Any
import pandas as pd
import os
import hmac
import hashlib

class BillingTier:
    FREE = "free"
    PREMIUM = "premium"
    ENTERPRISE = "enterprise"

class CostCalculator:
    MONTE_CARLO_COST = 0.15  # per simulation
    BASE_COST = 0.02        # per standard simulation
    
    @staticmethod
    def calculate(mode: str, duration: float, tier: str) -> float:
        if mode == 'MONTE_CARLO' and tier == BillingTier.PREMIUM:
            return CostCalculator.BASE_COST + (duration * CostCalculator.MONTE_CARLO_COST)
        return CostCalculator.BASE_COST

class UsageTracker:
    def __init__(self):
        self.usage_log = pd.DataFrame(columns=[
            'timestamp', 'user_id', 'tier', 'mode',
            'duration', 'cost', 'resource_usage'
        ])
        
    def record_usage(self, user_id: str, tier: str, mode: str, 
                    duration: float, resource_usage: Dict[str, Any]):
        cost = CostCalculator.calculate(mode, duration, tier)
        
        new_entry = pd.DataFrame([{
            'timestamp': datetime.utcnow(),
            'user_id': user_id,
            'tier': tier,
            'mode': mode,
            'duration': duration,
            'cost': cost,
            'resource_usage': str(resource_usage)
        }])
        
        self.usage_log = pd.concat([self.usage_log, new_entry], ignore_index=True)
        return cost

class BillingManager:
    def __init__(self):
        self.usage_tracker = UsageTracker()
        self._premium_key = os.environ.get('PREMIUM_API_KEY', '')
        
    def validate_premium_access(self, api_key: str) -> bool:
        """Validate premium API key using HMAC"""
        return hmac.compare_digest(
            api_key,
            self._premium_key
        )
    
    def get_user_tier(self, api_key: str) -> str:
        """Determine user's service tier"""
        if self.validate_premium_access(api_key):
            return BillingTier.PREMIUM
        return BillingTier.FREE
    
    def record_simulation(self, user_id: str, api_key: str, 
                         mode: str, duration: float, 
                         resource_usage: Dict[str, Any]) -> float:
        """Record simulation usage and return cost"""
        tier = self.get_user_tier(api_key)
        return self.usage_tracker.record_usage(
            user_id, tier, mode, duration, resource_usage
        )
        
    def get_monthly_usage(self, user_id: str) -> pd.DataFrame:
        """Get monthly usage statistics for a user"""
        return self.usage_log[
            self.usage_log['user_id'] == user_id
        ].copy()
