"""Data management and quality monitoring for research agent."""

import pandas as pd
import numpy as np
import pyarrow.parquet as pq
import pandera as pa
from datetime import datetime, timezone, timedelta
from typing import Dict, Optional, List, Any, Union
from pathlib import Path
import json
import asyncio
import aiohttp
from app.core.event_bus import Event, EventType
import logging
import redis


class MarketDataSchema(pa.DataFrameSchema):
    """Schema for market data validation."""

    def __init__(self):
        super().__init__(
            {
                "timestamp": pa.Column(pd.DatetimeTZDtype("ns", "UTC")),
                "symbol": pa.Column(str),
                "open": pa.Column(float, checks=pa.Check.greater_than(0)),
                "high": pa.Column(float, checks=pa.Check.greater_than(0)),
                "low": pa.Column(float, checks=pa.Check.greater_than(0)),
                "close": pa.Column(float, checks=pa.Check.greater_than(0)),
                "volume": pa.Column(float, checks=pa.Check.greater_than_or_equal_to(0)),
                "vwap": pa.Column(
                    float, checks=pa.Check.greater_than(0), nullable=True
                ),
                "trades": pa.Column(
                    int, checks=pa.Check.greater_than_or_equal_to(0), nullable=True
                ),
            }
        )


class DataLakeManager:
    """Manages data storage and retrieval in the data lake."""

    def __init__(self, base_path: str = "memory-bank/data"):
        self.base_path = Path(base_path)
        self.schema = MarketDataSchema()
        self._setup_directories()

    def _setup_directories(self):
        """Create necessary directories if they don't exist."""
        for dir_name in ["raw", "processed", "features", "models"]:
            (self.base_path / dir_name).mkdir(parents=True, exist_ok=True)

    def store_raw_data(
        self, df: pd.DataFrame, data_source: str, version: Optional[str] = None
    ) -> str:
        """Store raw market data with validation."""
        # Validate data against schema
        self.schema.validate(df)

        # Generate version if not provided
        if version is None:
            version = datetime.now(timezone.utc).strftime("%Y%m%d%H%M")

        # Store metadata
        metadata = {
            "data_source": data_source,
            "version": version,
            "rows": len(df),
            "symbols": df["symbol"].unique().tolist(),
            "start_time": df["timestamp"].min().isoformat(),
            "end_time": df["timestamp"].max().isoformat(),
        }

        # Save data and metadata
        file_path = self.base_path / f"raw/{data_source}_{version}.parquet"
        df.to_parquet(file_path, engine="pyarrow", compression="snappy", index=False)

        metadata_path = file_path.with_suffix(".json")
        with open(metadata_path, "w") as f:
            json.dump(metadata, f, indent=2)

        return str(file_path)

    def load_processed_data(
        self, pattern: str = "processed/*.parquet", columns: Optional[List[str]] = None
    ) -> pd.DataFrame:
        """Load processed data matching the pattern."""
        dataset = pq.ParquetDataset(self.base_path / pattern)
        if columns:
            return dataset.read(columns=columns).to_pandas()
        return dataset.read_pandas().to_pandas()


class DataQualityMonitor:
    """Monitors data quality and detects anomalies."""

    def __init__(self, event_bus=None):
        self.event_bus = event_bus
        self.anomaly_thresholds = {
            "price_spike": 0.15,  # 15% price change
            "volume_surge": 3.0,  # 3x normal volume
            "spread_widening": 0.02,  # 2% spread
            "trade_halt": 300,  # 5 minutes without trades
        }

    def detect_anomalies(self, df: pd.DataFrame) -> List[Dict]:
        """Detect various market anomalies in the data."""
        alerts = []
        grouped = df.groupby("symbol")

        for symbol, group in grouped:
            # Calculate metrics
            returns = group["close"].pct_change().dropna()
            vol_ratio = group["volume"] / group["volume"].rolling(20).mean()
            spread = (group["high"] - group["low"]) / group["close"]
            time_gaps = group["timestamp"].diff().dt.total_seconds()

            # Check for anomalies
            if any(returns.abs() > self.anomaly_thresholds["price_spike"]):
                alert = {
                    "type": "PRICE_SPIKE",
                    "symbol": symbol,
                    "timestamp": group.iloc[returns.abs().idxmax()]["timestamp"],
                    "value": returns.abs().max(),
                }
                alerts.append(alert)

            if any(vol_ratio > self.anomaly_thresholds["volume_surge"]):
                alert = {
                    "type": "VOLUME_SURGE",
                    "symbol": symbol,
                    "timestamp": group.iloc[vol_ratio.idxmax()]["timestamp"],
                    "value": vol_ratio.max(),
                }
                alerts.append(alert)

            if any(spread > self.anomaly_thresholds["spread_widening"]):
                alert = {
                    "type": "SPREAD_WIDENING",
                    "symbol": symbol,
                    "timestamp": group.iloc[spread.idxmax()]["timestamp"],
                    "value": spread.max(),
                }
                alerts.append(alert)

            if any(time_gaps > self.anomaly_thresholds["trade_halt"]):
                alert = {
                    "type": "TRADE_HALT",
                    "symbol": symbol,
                    "timestamp": group.iloc[time_gaps.idxmax()]["timestamp"],
                    "value": time_gaps.max(),
                }
                alerts.append(alert)

        # Publish alerts if event bus is available
        if self.event_bus and alerts:
            for alert in alerts:
                self.event_bus.publish(
                    Event(type=EventType.DATA_QUALITY_ALERT, payload=alert)
                )

        return alerts

    def calculate_quality_metrics(self, df: pd.DataFrame) -> Dict:
        """Calculate various data quality metrics."""
        metrics = {
            "completeness": {
                "missing_values": df.isnull().sum().to_dict(),
                "missing_rows": len(df.index) - len(df.dropna()),
            },
            "timeliness": {
                "max_delay": df["timestamp"].max() - pd.Timestamp.now(tz="UTC"),
                "avg_interval": df["timestamp"].diff().mean().total_seconds(),
            },
            "accuracy": {
                "price_zeros": (df[["open", "high", "low", "close"]] == 0)
                .sum()
                .to_dict(),
                "high_low_violations": (df["high"] < df["low"]).sum(),
                "negative_spreads": (df["high"] - df["low"] < 0).sum(),
            },
        }
        return metrics


class MarketDataCache:
    """Cache for market data with UTC timezone handling."""

    def __init__(self, redis_client=None):
        self.data: Dict[str, pd.DataFrame] = {}
        self.redis_client = redis_client
        self.logger = logging.getLogger(__name__)

    def get_data(self, symbol: str, lookback: Optional[int] = None) -> pd.DataFrame:
        """Get market data for symbol with optional lookback period."""
        if symbol not in self.data:
            raise KeyError(f"No data found for symbol {symbol}")

        df = self.data[symbol]
        if not df.index.tz:
            df.index = df.index.tz_localize(timezone.utc)

        if lookback:
            df = df.tail(lookback)

        return df

    def update_data(self, symbol: str, data: Union[pd.DataFrame, Dict]) -> None:
        """Update market data for symbol."""
        if isinstance(data, dict):
            data = pd.DataFrame.from_dict(data)

        # Ensure UTC timezone
        if not data.index.tz:
            data.index = data.index.tz_localize(timezone.utc)

        self.data[symbol] = data

    def slice_time(self, symbol: str, start: datetime, end: datetime) -> pd.DataFrame:
        """Get market data slice between start and end times."""
        if symbol not in self.data:
            raise KeyError(f"No data found for symbol {symbol}")

        df = self.data[symbol]

        # Convert times to UTC if needed
        if not start.tzinfo:
            start = start.replace(tzinfo=timezone.utc)
        if not end.tzinfo:
            end = end.replace(tzinfo=timezone.utc)

        return df[start:end]

    async def persist(self) -> None:
        """Persist cache to Redis if available."""
        if not self.redis_client:
            return

        try:
            for symbol, df in self.data.items():
                key = f"market_data:{symbol}"
                # Convert to JSON with orient='split' to preserve index
                await self.redis_client.set(
                    key, df.to_json(orient="split", date_format="iso")
                )
        except Exception as e:
            self.logger.error(f"Failed to persist cache: {str(e)}")

    async def load(self, symbol: str) -> None:
        """Load market data from Redis if available."""
        if not self.redis_client:
            return

        try:
            key = f"market_data:{symbol}"
            data = await self.redis_client.get(key)
            if data:
                # Read JSON with orient='split' to preserve index
                df = pd.read_json(data, orient="split")
                if not df.index.tz:
                    df.index = df.index.tz_localize(timezone.utc)
                self.update_data(symbol, df)
        except Exception as e:
            self.logger.error(f"Failed to load cache: {str(e)}")
