"""Event bus system for backtesting infrastructure."""

from __future__ import annotations

import asyncio
import json
import logging
import hmac
import hashlib
from datetime import datetime, timezone
from typing import Any, Dict, List, Optional, Callable, Awaitable, Union
from enum import Enum
import redis.asyncio as redis
from concurrent.futures import ThreadPoolExecutor
import pandas as pd
import numpy as np
import uuid
import os


class EventType(str, Enum):
    """Event types for the system."""

    STRATEGY_SUBMITTED = "strategy.submitted"
    STRATEGY_STARTED = "strategy.started"
    STRATEGY_COMPLETED = "strategy.completed"
    STRATEGY_FAILED = "strategy.failed"
    AGENT_REQUEST = "agent.request"
    AGENT_RESPONSE = "agent.response"
    DATA_UPDATED = "data.updated"
    SYSTEM_ALERT = "system.alert"
    MARKET_DATA = "market.data"
    ORDER_BOOK = "market.orderbook"
    TRADE = "market.trade"
    STRATEGY_UPDATE = "strategy.update"
    MODEL_UPDATE = "model.update"
    KNOWLEDGE_UPDATE = "knowledge.update"
    ANOMALY_DETECTED = "market.anomaly"


class EventEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, datetime):
            return obj.isoformat()
        if isinstance(obj, EventType):
            return obj.value
        return super().default(obj)


class Event:
    """Event message structure."""

    def __init__(
        self,
        type: EventType,
        payload: Dict[str, Any],
        source: str,
        trace_id: Optional[str] = None,
        timestamp: Optional[datetime] = None,
    ):
        """Initialize event.

        Args:
            type: Event type
            payload: Event data
            source: Source of the event (e.g., 'strategy', 'market', 'system')
            trace_id: Optional trace ID for event correlation
            timestamp: Optional event timestamp (UTC)
        """
        self.type = type
        self.payload = payload
        self.source = source
        self.trace_id = trace_id or str(uuid.uuid4())
        self.timestamp = timestamp or datetime.now(timezone.utc)

    def to_dict(self) -> Dict[str, Any]:
        """Convert event to dictionary."""
        return {
            "type": self.type,
            "payload": self.payload,
            "source": self.source,
            "trace_id": self.trace_id,
            "timestamp": self.timestamp,
        }

    def to_json(self) -> str:
        """Convert event to JSON string."""
        return json.dumps(self.to_dict(), cls=EventEncoder)

    def encode(self) -> bytes:
        """Encode event for signing."""
        return self.to_json().encode("utf-8")

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "Event":
        """Create event from dictionary."""
        return cls(
            type=EventType(data["type"]),
            payload=data["payload"],
            source=data["source"],
            trace_id=data.get("trace_id"),
            timestamp=datetime.fromisoformat(data["timestamp"]),
        )

    @classmethod
    def from_json(cls, json_str: str) -> "Event":
        """Decode event from JSON string."""
        data = json.loads(json_str)
        return cls.from_dict(data)


class EventBus:
    """Distributed event bus using Redis Streams."""

    def __init__(
        self,
        redis_url: str = None,
        stream_name: str = "events",
        consumer_group: str = "backtester",
        hmac_key: Optional[bytes] = None,
        redis_client: Optional[redis.Redis] = None,
    ):
        """Initialize event bus with Redis connection."""
        self.redis_url = redis_url or os.getenv('REDIS_URL', 'redis://localhost:6379')
        self.redis_client = self._create_redis_client()
        self.stream_name = stream_name
        self.consumer_group = consumer_group
        self.hmac_key = hmac_key or os.getenv("EVENT_BUS_KEY", "default_key").encode(
            "utf-8"
        )
        self.logger = logging.getLogger(__name__)

        self.handlers: Dict[EventType, List[Callable]] = {
            event_type: [] for event_type in EventType
        }
        self.executor = ThreadPoolExecutor()
        self.processing = False

    def _create_redis_client(self):
        """Create Redis client."""
        retry_count = 0
        max_retries = 3
        while retry_count < max_retries:
            try:
                return redis.from_url(self.redis_url)
            except Exception as e:
                retry_count += 1
                self.logger.warning(f"Redis connection attempt {retry_count} failed: {e}")
                if retry_count == max_retries:
                    self.logger.error("Failed to connect to Redis after max retries")
                    raise
                asyncio.sleep(1)

    async def connect(self):
        """Connect to Redis and create consumer group."""
        try:
            await self.redis_client.xgroup_create(
                name=self.stream_name, groupname=self.consumer_group, mkstream=True
            )
        except redis.ResponseError as e:
            if "BUSYGROUP" not in str(e):
                raise

    async def aclose(self):
        """Close Redis connection."""
        await self.redis_client.aclose()

    def _sign_event(self, event: Event) -> str:
        """Create HMAC signature for event."""
        h = hmac.new(self.hmac_key, event.encode(), hashlib.sha256)
        return h.hexdigest()

    def _verify_signature(self, event: Event, signature: str) -> bool:
        """Verify HMAC signature of event."""
        expected = self._sign_event(event)
        return hmac.compare_digest(signature, expected)

    async def publish(self, event: Union[Event, Dict[str, Any]]):
        """Publish event to Redis stream."""
        if isinstance(event, dict):
            # Convert dict to Event
            event = Event(
                type=EventType(event["type"]),
                payload=event.get("payload", {}),
                timestamp=datetime.fromisoformat(
                    event.get("timestamp", datetime.now(timezone.utc).isoformat())
                ),
                trace_id=event.get("trace_id"),
                source=event.get("source"),
            )

        event_json = event.to_json()
        signature = self._sign_event(event)

        # Add to stream with signature
        await self.redis_client.xadd(
            name=self.stream_name,
            fields={
                "data": event_json,
                "signature": signature,
                "timestamp": event.timestamp.isoformat(),
            },
        )

        self.logger.debug(f"Published event: {event.type}")

    async def subscribe(
        self, event_type: EventType, handler: Callable[[Event, str], Awaitable[None]]
    ):
        """Subscribe to events of specific type."""
        self.handlers[event_type].append(handler)

    async def start_processing(self):
        """Start processing events from stream."""
        self.processing = True
        try:
            while self.processing:
                try:
                    # Read new messages
                    messages = await self.redis_client.xreadgroup(
                        groupname=self.consumer_group,
                        consumername="consumer",
                        streams={self.stream_name: ">"},
                        count=10,
                        block=1000,
                    )

                    if not messages:
                        continue

                    # Process messages
                    for _, message_list in messages:
                        for message_id, message in message_list:
                            event_data = message[b"data"].decode()
                            signature = message[b"signature"].decode()

                            # Verify signature
                            event = Event.from_json(event_data)
                            if self._verify_signature(event, signature):
                                # Parse event
                                # Call handlers
                                handlers = self.handlers[event.type]
                                for handler in handlers:
                                    try:
                                        await handler(event, signature)
                                    except Exception as e:
                                        self.logger.error(f"Handler error: {e}")
                            else:
                                self.logger.error(
                                    f"Invalid signature for message {message_id}"
                                )

                            # Acknowledge message
                            await self.redis_client.xack(
                                self.stream_name, self.consumer_group, message_id
                            )

                except Exception as e:
                    self.logger.error(f"Event processing error: {e}")
                    await asyncio.sleep(1)

        except asyncio.CancelledError:
            self.processing = False
            raise


# Example usage
async def example_handler(event: Event, signature: str):
    """Example event handler."""
    print(f"Received event: {event.type}")
    print(f"Payload: {event.payload}")


async def main():
    """Example usage of EventBus."""
    # Initialize event bus
    bus = EventBus()
    await bus.connect()

    # Subscribe to events
    await bus.subscribe(EventType.STRATEGY_SUBMITTED, example_handler)

    # Start processing in background
    process_task = asyncio.create_task(bus.start_processing())

    # Publish some events
    await bus.publish(
        Event(
            type=EventType.STRATEGY_SUBMITTED,
            payload={"strategy_id": "123", "name": "MA Crossover"},
            source="example",
        )
    )

    # Wait a bit
    await asyncio.sleep(5)

    # Cleanup
    process_task.cancel()
    await bus.aclose()


if __name__ == "__main__":
    asyncio.run(main())
