"""Parallel backtesting and ML model execution engine."""

from __future__ import annotations
from typing import List, Dict, Any, Optional, Callable, Awaitable
import numpy as np
import pandas as pd
import asyncio
from joblib import Parallel, delayed
from datetime import datetime, timezone
from dataclasses import dataclass


@dataclass
class BacktestResult:
    strategy_id: str
    returns: np.ndarray
    positions: np.ndarray
    metrics: Dict[str, float]
    parameters: Dict[str, Any]
    timestamp: datetime = datetime.now(timezone.utc)


class ParallelEngine:
    """Parallel execution engine for ML models and backtesting"""

    def __init__(self, n_jobs: int = -1, batch_size: int = 1000):
        self.n_jobs = n_jobs
        self.batch_size = batch_size
        self._executor = None

    async def execute(self, func: Callable, *args, **kwargs) -> Any:
        """Execute a function in parallel using joblib

        Args:
            func: Function to execute
            *args: Positional arguments for the function
            **kwargs: Keyword arguments for the function

        Returns:
            Result of the function execution
        """
        loop = asyncio.get_event_loop()
        return await loop.run_in_executor(self._executor, lambda: func(*args, **kwargs))

    def run_parallel(self, func: Callable, items: List[Any]) -> List[Any]:
        """Run a function over a list of items in parallel

        Args:
            func: Function to execute
            items: List of items to process

        Returns:
            List of results
        """
        return Parallel(n_jobs=self.n_jobs)(delayed(func)(item) for item in items)


class ParallelBacktest:
    """Parallel backtesting engine using joblib"""

    def __init__(self, n_jobs: int = -1, batch_size: int = 1000):
        self.n_jobs = n_jobs
        self.batch_size = batch_size

    def run(self, strategies: List[Dict], data: pd.DataFrame) -> List[BacktestResult]:
        """Run multiple strategies in parallel

        Args:
            strategies: List of strategy configurations
            data: Market data as pandas DataFrame with UTC index

        Returns:
            List of BacktestResult objects
        """
        # Split strategies into batches
        strategy_batches = [
            strategies[i : i + self.batch_size]
            for i in range(0, len(strategies), self.batch_size)
        ]

        # Process batches in parallel
        results = Parallel(n_jobs=self.n_jobs)(
            delayed(self._process_batch)(batch, data.copy())
            for batch in strategy_batches
        )

        # Flatten results
        return [result for batch_results in results for result in batch_results]

    def _process_batch(
        self, strategies: List[Dict], data: pd.DataFrame
    ) -> List[BacktestResult]:
        """Process a batch of strategies

        Args:
            strategies: List of strategy configurations
            data: Market data

        Returns:
            List of BacktestResult objects
        """
        results = []
        for strategy in strategies:
            result = self._run_strategy(strategy, data)
            results.append(result)
        return results

    def _run_strategy(self, strategy: Dict, data: pd.DataFrame) -> BacktestResult:
        """Run a single strategy

        Args:
            strategy: Strategy configuration
            data: Market data

        Returns:
            BacktestResult object
        """
        # Initialize arrays
        n = len(data)
        returns = np.zeros(n)
        positions = np.zeros(n)

        # Run strategy
        for i in range(n):
            row = data.iloc[i]
            signal = strategy["logic"](row)
            positions[i] = signal

            if i > 0:
                returns[i] = (
                    positions[i - 1]
                    * (row["close"] - data.iloc[i - 1]["close"])
                    / data.iloc[i - 1]["close"]
                )

        # Calculate metrics
        metrics = {
            "total_return": float(np.sum(returns)),
            "sharpe_ratio": float(
                np.mean(returns) / np.std(returns) if np.std(returns) > 0 else 0
            ),
            "max_drawdown": float(self._calculate_max_drawdown(returns)),
        }

        return BacktestResult(
            strategy_id=strategy["id"],
            returns=returns,
            positions=positions,
            metrics=metrics,
            parameters=strategy.get("parameters", {}),
        )

    def _calculate_max_drawdown(self, returns: np.ndarray) -> float:
        """Calculate maximum drawdown from returns"""
        cumulative = np.cumprod(1 + returns)
        running_max = np.maximum.accumulate(cumulative)
        drawdowns = cumulative / running_max - 1
        return float(np.min(drawdowns))
