from __future__ import annotations
from typing import Optional, Dict
from dataclasses import dataclass
import numpy as np
import pandas as pd


@dataclass
class PositionSizingConfig:
    mode: str = "kelly"
    max_risk: float = 0.02
    volatility_window: int = 20
    kelly_leverage_limit: float = 2.0


class RiskManager:
    def __init__(self, initial_capital: float = 100000.0):
        self.capital = initial_capital
        self.position_history: pd.DataFrame = pd.DataFrame(
            columns=["timestamp", "size", "price", "symbol"]
        )

    def calculate_position_size(
        self,
        current_price: float,
        volatility: float,
        config: PositionSizingConfig,
        win_rate: Optional[float] = None,
        win_loss_ratio: Optional[float] = None,
    ) -> float:
        """Calculate position size using selected risk management method"""
        if config.mode == "fixed":
            return self._fixed_fraction(config.max_risk)
        elif config.mode == "kelly":
            return self._kelly_criterion(
                win_rate, win_loss_ratio, config.kelly_leverage_limit
            )
        elif config.mode == "volatility":
            return self._volatility_scaled(
                current_price, volatility, config.volatility_window
            )
        else:
            raise ValueError(f"Unknown risk mode: {config.mode}")

    def _fixed_fraction(self, risk_per_trade: float) -> float:
        return self.capital * risk_per_trade

    def _kelly_criterion(
        self, win_rate: float, win_loss_ratio: float, leverage_limit: float
    ) -> float:
        kelly_f = (win_rate * win_loss_ratio - (1 - win_rate)) / win_loss_ratio
        return min(kelly_f, leverage_limit) * self.capital

    def _volatility_scaled(self, price: float, volatility: float, window: int) -> float:
        if volatility == 0:
            return 0.0
        return (self.capital * 0.1) / (volatility * np.sqrt(window))

    def update_capital(self, pnl: float):
        self.capital += pnl
        self.capital = max(self.capital, 0)

    def get_current_exposure(self) -> Dict[str, float]:
        """Get current market exposure by symbol"""
        if self.position_history.empty:
            return {}

        latest = self.position_history.iloc[-1]
        return {latest["symbol"]: latest["size"] * latest["price"]}
