"""Security module for strategy validation."""

import hmac
import hashlib
import json
import logging
import os
from typing import Dict, Any
from datetime import datetime, timezone, timedelta


class SecurityException(Exception):
    """Custom exception for security-related errors."""

    pass


class MCPVaultIntegration:
    """Implements secure key management per MEMORY[user_global]."""

    def __init__(self):
        self.logger = logging.getLogger(__name__)
        self._keys = {}
        self._last_rotation = {}
        self.ROTATION_INTERVAL = timedelta(hours=24)  # MCP_VAULT_ROTATE_24H

    def get_key(self, service: str) -> str:
        """Get API key for service with automatic rotation."""
        now = datetime.now(timezone.utc)

        # Check if key needs rotation
        if (
            service not in self._keys
            or service not in self._last_rotation
            or now - self._last_rotation[service] > self.ROTATION_INTERVAL
        ):
            self._rotate_key(service)

        return self._keys[service]

    def _rotate_key(self, service: str) -> None:
        """Rotate API key for service."""
        env_key = f"{service.upper()}_KEY"
        key = os.environ.get(env_key)
        if not key:
            raise SecurityException(f"Missing {env_key} environment variable")

        self._keys[service] = key
        self._last_rotation[service] = datetime.now(timezone.utc)
        self.logger.info(f"Rotated API key for {service}")


class MCPAuthGuard:
    """Implements MEMORY[user_global] security requirements."""

    def __init__(self):
        """Initialize the security guard."""
        self.logger = logging.getLogger(__name__)
        self.PARAM_BOUNDS = {
            "risk_per_trade": (0.01, 0.05),
            "position_size": (0.1, 2.0),
            "stop_loss_pct": (0.01, 0.1),
            "take_profit_pct": (0.02, 0.2),
        }

    def _get_signing_key(self) -> str:
        """Get the strategy signing key."""
        # Get key from environment variable
        key = os.environ.get("ROSE_RESEARCH_KEY")
        if not key:
            raise SecurityException("Missing ROSE_RESEARCH_KEY environment variable")
        return key

    def verify_strategy_signature(self, strategy_dna: dict):
        """Verify HMAC signature of strategy parameters"""
        try:
            signature = strategy_dna.get("signature")
            if not signature:
                raise SecurityException("Missing strategy signature")

            # Verify HMAC signature
            key = self._get_signing_key()
            computed_signature = hmac.new(
                key.encode(),
                json.dumps(strategy_dna["parameters"], sort_keys=True).encode(),
                hashlib.sha256,
            ).hexdigest()

            if not hmac.compare_digest(signature, computed_signature):
                raise SecurityException("Invalid strategy signature")

            return strategy_dna

        except Exception as e:
            self.logger.error(f"Strategy verification failed: {str(e)}")
            raise SecurityException(f"Strategy verification failed: {str(e)}")

    def validate_optimized_params(self, parameters: Dict[str, Any]):
        """Validate optimized parameters against security bounds"""
        try:
            for param_name, value in parameters.items():
                if param_name in self.PARAM_BOUNDS:
                    min_val, max_val = self.PARAM_BOUNDS[param_name]
                    if (
                        not isinstance(value, (int, float))
                        or value < min_val
                        or value > max_val
                    ):
                        raise SecurityException(
                            f"Parameter {param_name} value {value} outside allowed range [{min_val}, {max_val}]"
                        )

            # Check for dangerous combinations
            if (
                parameters.get("position_size", 1.0) > 1.5
                and parameters.get("stop_loss_pct", 0.05) < 0.03
            ):
                raise SecurityException("Dangerous parameter combination detected")

        except Exception as e:
            self.logger.error(f"Parameter validation failed: {str(e)}")
            raise SecurityException(f"Parameter validation failed: {str(e)}")
