from fastapi import HTTPException, Security
from fastapi.security import HTTPBearer, HTTPAuthorizationCredentials
from typing import Optional, Dict, Callable, Any, List
from datetime import datetime, timedelta, timezone
import jwt
import hmac
import hashlib
from functools import wraps
from pytz import UTC

security = HTTPBearer()


class MCPAuthGuard:
    SECRET_KEY = (
        "your-secret-key"  # In production, this should be in environment variables
    )
    ALGORITHM = "HS256"

    @staticmethod
    def create_token(
        user_id: str, permissions: List[str], expires_delta: Optional[timedelta] = None
    ) -> str:
        """Create a new JWT token."""
        if expires_delta:
            expire = datetime.now(timezone.utc) + expires_delta
        else:
            expire = datetime.now(timezone.utc) + timedelta(minutes=15)

        to_encode = {
            "sub": user_id,
            "permissions": permissions,
            "exp": expire.timestamp(),
            "iat": datetime.now(timezone.utc).timestamp(),
        }

        return jwt.encode(
            to_encode, MCPAuthGuard.SECRET_KEY, algorithm=MCPAuthGuard.ALGORITHM
        )

    @staticmethod
    def verify_token(token: str) -> Dict:
        """Verify a JWT token."""
        try:
            payload = jwt.decode(
                token, MCPAuthGuard.SECRET_KEY, algorithms=[MCPAuthGuard.ALGORITHM]
            )
            return payload
        except jwt.PyJWTError:
            raise ValueError("Invalid authentication token")

    @staticmethod
    def check_perms(required_permission: str):
        async def permission_dependency(
            credentials: HTTPAuthorizationCredentials = Security(security),
        ) -> Dict[str, Any]:
            try:
                token = credentials.credentials
                payload = MCPAuthGuard.verify_token(token)

                # Check if user has required permission
                if required_permission not in payload.get("permissions", []):
                    raise HTTPException(
                        status_code=403,
                        detail=f"Permission denied: {required_permission} required",
                    )

                return payload

            except ValueError:
                raise HTTPException(
                    status_code=401, detail="Could not validate credentials"
                )

        return permission_dependency

    @staticmethod
    async def websocket_auth(websocket):
        """Authenticate a WebSocket connection."""
        try:
            token = websocket.headers.get("Authorization", "").replace("Bearer ", "")
            if not token:
                await websocket.close(code=4001, reason="Missing authentication token")
                return False

            MCPAuthGuard.verify_token(token)
            return True

        except ValueError as e:
            await websocket.close(code=4001, reason=str(e))
            return False

    @staticmethod
    def sign_strategy(strategy_code: str, user_id: str) -> str:
        """Sign strategy code with HMAC-SHA256"""
        key = f"ROSE_RESEARCH_KEY_{user_id}"  # In production, this should be from vault
        signature = hmac.new(
            key.encode(), strategy_code.encode(), hashlib.sha256
        ).hexdigest()
        return signature

    @staticmethod
    def verify_strategy_signature(
        strategy_code: str, signature: str, user_id: str
    ) -> bool:
        """Verify strategy signature"""
        expected_signature = MCPAuthGuard.sign_strategy(strategy_code, user_id)
        return hmac.compare_digest(signature, expected_signature)

    @staticmethod
    def requires_auth(func: Callable):
        """Decorator for WebSocket authentication"""

        @wraps(func)
        async def wrapper(websocket, *args, **kwargs):
            try:
                token = websocket.headers.get("Authorization", "").replace(
                    "Bearer ", ""
                )
                if not token:
                    await websocket.close(
                        code=4001, reason="Missing authentication token"
                    )
                    return

                payload = MCPAuthGuard.verify_token(token)

                # Add user info to websocket state
                websocket.state.user = payload
                return await func(websocket, *args, **kwargs)

            except ValueError:
                await websocket.close(code=4001, reason="Invalid authentication token")
                return
