"""DSPy-powered Strategy Optimizer"""

from dspy import Module, Signature, Predict
from typing import Dict, Any, ClassVar
from dspy.signatures import Signature
from dspy.signatures.field import InputField, OutputField
import pandas as pd
from datetime import datetime, timezone
import json
import logging
from pydantic import ValidationError


class StrategyOptimizationSignature(Signature):
    """Optimize trading strategy parameters based on market conditions"""

    market_analysis: str = InputField(desc="Technical analysis of current market state")
    current_params: Dict[str, Any] = InputField(
        desc="Current strategy parameters as JSON"
    )
    optimized_params: Dict[str, Any] = OutputField(
        desc="Optimized parameters JSON with risk adjustments"
    )


class StrategyOptimizer(Module):
    def __init__(self):
        super().__init__()
        self.predict = Predict(StrategyOptimizationSignature)
        self.logger = logging.getLogger(__name__)

    def forward(
        self, market_data: pd.DataFrame, strategy_params: Dict[str, Any]
    ) -> Dict[str, Any]:
        """Optimize strategy parameters using market data"""
        if market_data.empty or not isinstance(strategy_params, dict):
            self.logger.warning("Invalid input data for optimization")
            return strategy_params

        try:
            analysis = self._generate_market_analysis(market_data)
            result = self.predict(
                market_analysis=analysis, current_params=json.dumps(strategy_params)
            ).optimized_params

            optimized = self.parse_optimizer_output(result)
            if not optimized:
                self.logger.warning("Optimization failed, using original parameters")
                return strategy_params

            return optimized

        except Exception as e:
            self.logger.error(f"Optimization error: {str(e)}")
            return strategy_params

    def _generate_market_analysis(self, data: pd.DataFrame) -> str:
        """Generate market analysis with validation"""
        try:
            last_row = data.iloc[-1]
            metrics = {
                "trend": last_row.get("trend", "unknown"),
                "volatility": float(last_row.get("volatility", 0)),
                "volume": float(last_row.get("volume", 0)),
                "rsi": float(last_row.get("rsi", 50)),
            }

            return (
                f"Market Analysis - "
                f"Trend: {metrics['trend']}, "
                f"Volatility: {metrics['volatility']:.2f}, "
                f"Volume: {metrics['volume']:.0f}, "
                f"RSI: {metrics['rsi']:.1f}"
            )
        except Exception as e:
            self.logger.error(f"Market analysis generation failed: {str(e)}")
            return "Market analysis unavailable"

    def parse_optimizer_output(self, optimizer_output: str) -> dict:
        try:
            parsed = json.loads(optimizer_output)
            if not all(k in parsed for k in ("parameters", "metrics")):
                raise ValueError("Invalid optimizer output structure")
            return parsed
        except json.JSONDecodeError as e:
            self.logger.error(f"JSON parsing failed: {e}")
            raise ValidationError("Invalid optimizer output format") from e
        except KeyError as e:
            self.logger.error(f"Missing required field: {e}")
            raise ValidationError("Invalid optimizer output structure") from e
