"""Strategy execution and optimization service."""

from typing import Dict, Any, Optional, List
from app.core.strategy_optimizer import StrategyOptimizer
from app.core.security import MCPAuthGuard, SecurityException
from app.core.market_cache import MarketCache
from datetime import datetime, timezone
import pandas as pd
import json
import logging
import os
from .parallel_engine import ParallelBacktest, BacktestResult


class StrategyOrchestrator(MCPAuthGuard):
    """Orchestrates strategy execution and optimization."""

    def __init__(self):
        super().__init__()
        self.optimizer = StrategyOptimizer()
        self.market_cache = MarketCache()
        self.parallel_engine = ParallelBacktest()
        self.data_cache = MarketDataCache()
        self.logger = logging.getLogger(__name__)

        # Required fields per MEMORY[f656605e]
        self.required_fields = set(
            ["id", "symbol", "parameters", "signature", "metrics"]
        )

        # Create results directory if it doesn't exist
        os.makedirs("results", exist_ok=True)

    async def execute_strategy(self, strategy_dna: Dict[str, Any]) -> Dict[str, Any]:
        """Execute strategy with DSPy optimization"""
        try:
            # Verify strategy signature per MEMORY[user_global]
            signed_dna = self.verify_strategy_signature(
                signature=strategy_dna.get("signature", ""),
                user_id=strategy_dna.get("user_id", ""),
            )
            if not signed_dna:
                raise SecurityException("Invalid strategy signature")

            # Get market data
            market_data = self.market_cache.get_data(
                symbol=signed_dna["symbol"], lookback=signed_dna.get("lookback", 100)
            )

            # Optimize parameters
            optimized_params = self.optimizer.forward(
                market_data=market_data, strategy_params=signed_dna["parameters"]
            )

            # Update strategy with optimized parameters
            signed_dna["parameters"].update(optimized_params)

            # Execute backtest with market data
            result = {
                "id": signed_dna["id"],
                "name": signed_dna["name"],
                "symbol": signed_dna["symbol"],
                "timeframe": signed_dna.get(
                    "timeframe", "1d"
                ),  # Default to daily timeframe
                "parameters": signed_dna["parameters"],
                "metrics": signed_dna.get("metrics", {}),
                "timestamp": datetime.now(timezone.utc).isoformat(),
            }

            # Store results per MEMORY[f656605e]
            results_file = f"results/{result['id']}.json"
            with open(results_file, "w") as f:
                json.dump(result, f, indent=2)

            return result

        except SecurityException as se:
            self.logger.error(f"Security validation failed: {str(se)}")
            raise
        except Exception as e:
            await self._handle_execution_error(e)
            raise

    def execute_strategies(self, strategies: List[Dict]) -> List[BacktestResult]:
        """Execute multiple strategies in parallel"""
        # Get data from cache
        data = self.data_cache.get_data(
            symbol=strategies[0].get("symbol"), timeframe=strategies[0].get("timeframe")
        )

        # Run parallel backtests
        results = self.parallel_engine.run(strategies, data)

        # Store results
        for result in results:
            self._store_results(result)

        return results

    def _store_results(self, result: BacktestResult):
        """Store backtest results"""
        # Convert to parquet format
        df = pd.DataFrame({"returns": result.returns, "positions": result.positions})

        # Save with UTC timestamp
        timestamp = datetime.now(timezone.utc).strftime("%Y%m%d_%H%M%S")
        filename = f"results/{result.strategy_id}_{timestamp}.parquet"
        df.to_parquet(filename)

    def _store_results(
        self, strategy_id: str, parameters: Dict[str, Any], metrics: Dict[str, Any]
    ):
        """Store strategy results with versioning"""
        try:
            # Create DataFrame for new result
            new_result = pd.DataFrame(
                [
                    {
                        "timestamp": datetime.now(timezone.utc),
                        "strategy_id": strategy_id,
                        "parameters": json.dumps(parameters),
                        "metrics": json.dumps(metrics),
                    }
                ]
            )

            # Try to read existing results
            results_file = "results/backtests.parquet"
            if os.path.exists(results_file):
                existing_results = pd.read_parquet(results_file)
                results = pd.concat([existing_results, new_result], ignore_index=True)
            else:
                results = new_result

            # Save updated results
            results.to_parquet(results_file, index=False)

        except Exception as e:
            self.logger.error(f"Failed to store results: {str(e)}")
            raise

    async def _handle_execution_error(self, error: Exception) -> None:
        """Handle strategy execution errors"""
        error_context = {
            "error_type": type(error).__name__,
            "error_message": str(error),
            "timestamp": datetime.now(timezone.utc).isoformat(),
        }
        self.logger.info(f"Error context: {json.dumps(error_context)}")

        if isinstance(error, SecurityException):
            raise error
        else:
            raise RuntimeError(f"Strategy execution failed: {str(error)}")


class RiskManager:
    def calculate_position_size(
        self, close: float, volatility: float, risk_config: Dict[str, Any]
    ) -> float:
        # TO DO: implement risk management logic
        return 1.0


class MarketDataCache:
    def get_data(self, symbol: str, timeframe: str) -> pd.DataFrame:
        # TO DO: implement market data cache logic
        return pd.DataFrame()
