"""Real-time data streaming components."""

import asyncio
import json
import logging
import websockets
import aiohttp
from collections import deque
from datetime import datetime, timezone
from sortedcontainers import SortedDict
from typing import Dict, List, Optional
from app.core.event_bus import Event, EventType


class MarketDataStream:
    """Handles real-time market data streaming."""

    def __init__(self, symbols: List[str], event_bus):
        self.symbols = symbols
        self.event_bus = event_bus
        self.order_books = {sym: SortedDict() for sym in symbols}
        self.trade_history = {sym: deque(maxlen=1000) for sym in symbols}
        self.logger = logging.getLogger(__name__)

    async def connect_coinbase(self):
        """Connect to Coinbase WebSocket feed."""
        uri = "wss://ws-feed.pro.coinbase.com"
        while True:
            try:
                async with websockets.connect(uri) as websocket:
                    # Subscribe to channels
                    await websocket.send(
                        json.dumps(
                            {
                                "type": "subscribe",
                                "product_ids": self.symbols,
                                "channels": ["level2", "matches", "ticker"],
                            }
                        )
                    )

                    # Process messages
                    async for message in websocket:
                        data = json.loads(message)
                        await self._process_message(data)

            except websockets.WebSocketException as e:
                self.logger.error(f"WebSocket error: {e}")
                await asyncio.sleep(5)  # Wait before reconnecting

            except Exception as e:
                self.logger.error(f"Unexpected error: {e}")
                await asyncio.sleep(5)

    async def _process_message(self, data: Dict):
        """Process incoming WebSocket messages."""
        try:
            if data["type"] == "snapshot":
                await self._handle_snapshot(data)
            elif data["type"] == "l2update":
                await self._handle_order_book_update(data)
            elif data["type"] == "match":
                await self._handle_trade(data)
            elif data["type"] == "ticker":
                await self._handle_ticker(data)

        except Exception as e:
            self.logger.error(f"Error processing message: {e}")

    async def _handle_snapshot(self, data: Dict):
        """Handle order book snapshot."""
        symbol = data["product_id"]
        self.order_books[symbol].clear()

        # Process bids and asks
        for price, size in data.get("bids", []):
            self.order_books[symbol][float(price)] = float(size)

        for price, size in data.get("asks", []):
            self.order_books[symbol][-float(price)] = float(size)  # Negative for asks

        await self.event_bus.publish(
            Event(
                type=EventType.MARKET_DATA_SNAPSHOT,
                payload={
                    "symbol": symbol,
                    "timestamp": datetime.now(timezone.utc).isoformat(),
                    "bids": data.get("bids", [])[:10],  # Top 10 levels
                    "asks": data.get("asks", [])[:10],
                },
            )
        )

    async def _handle_order_book_update(self, data: Dict):
        """Handle order book updates."""
        symbol = data["product_id"]
        for side, price, size in data.get("changes", []):
            price = float(price)
            size = float(size)

            if side == "buy":
                if size == 0:
                    self.order_books[symbol].pop(price, None)
                else:
                    self.order_books[symbol][price] = size
            else:  # sell
                if size == 0:
                    self.order_books[symbol].pop(-price, None)
                else:
                    self.order_books[symbol][-price] = size

        # Publish top of book
        bids = [(k, v) for k, v in self.order_books[symbol].items() if k > 0][-10:]
        asks = [(-k, v) for k, v in self.order_books[symbol].items() if k < 0][:10]

        await self.event_bus.publish(
            Event(
                type=EventType.ORDER_BOOK_UPDATE,
                payload={
                    "symbol": symbol,
                    "timestamp": datetime.now(timezone.utc).isoformat(),
                    "bids": bids,
                    "asks": asks,
                },
            )
        )

    async def _handle_trade(self, data: Dict):
        """Handle individual trades."""
        symbol = data["product_id"]
        trade = {
            "price": float(data["price"]),
            "size": float(data["size"]),
            "side": data["side"],
            "timestamp": data["time"],
        }
        self.trade_history[symbol].append(trade)

        await self.event_bus.publish(
            Event(
                type=EventType.TRADE_UPDATE, payload={"symbol": symbol, "trade": trade}
            )
        )

    async def _handle_ticker(self, data: Dict):
        """Handle ticker updates."""
        await self.event_bus.publish(
            Event(
                type=EventType.TICKER_UPDATE,
                payload={
                    "symbol": data["product_id"],
                    "price": float(data["price"]),
                    "bid": float(data["best_bid"]),
                    "ask": float(data["best_ask"]),
                    "volume": float(data["volume_24h"]),
                    "timestamp": data["time"],
                },
            )
        )


class NewsStream:
    """Handles real-time news and social media streaming."""

    def __init__(self, event_bus):
        self.event_bus = event_bus
        self.logger = logging.getLogger(__name__)

    async def monitor_rss(self, feeds: List[str]):
        """Monitor RSS feeds for news updates."""
        async with aiohttp.ClientSession() as session:
            while True:
                try:
                    for feed in feeds:
                        async with session.get(feed) as response:
                            if response.status == 200:
                                content = await response.text()
                                await self._process_news(content)
                            else:
                                self.logger.warning(
                                    f"Failed to fetch {feed}: {response.status}"
                                )

                    await asyncio.sleep(60)  # Check every minute

                except Exception as e:
                    self.logger.error(f"Error monitoring RSS: {e}")
                    await asyncio.sleep(60)

    async def _process_news(self, content: str):
        """Process and publish news updates."""
        try:
            # Extract relevant information (simplified)
            news_items = self._parse_rss(content)

            for item in news_items:
                await self.event_bus.publish(
                    Event(type=EventType.NEWS_UPDATE, payload=item)
                )

        except Exception as e:
            self.logger.error(f"Error processing news: {e}")

    def _parse_rss(self, content: str) -> List[Dict]:
        """Parse RSS content into structured format."""
        # Placeholder for actual RSS parsing
        # In practice, use feedparser or similar library
        return [
            {
                "title": "Sample News",
                "content": "Content",
                "timestamp": datetime.now(timezone.utc).isoformat(),
            }
        ]
