from fastapi import WebSocket
from redis.asyncio import Redis
import json
import asyncio
from datetime import datetime, timezone
from typing import Dict, Any, Set
import os
import logging


class WebSocketManager:
    """Real-time market data and strategy execution bridge"""

    def __init__(self):
        self.redis = Redis.from_url(os.getenv("REDIS_URL", "redis://localhost:6379"))
        self.active_connections: Dict[str, Set[WebSocket]] = {
            "market_data": set(),
            "strategy_updates": set(),
        }
        self.logger = logging.getLogger(__name__)

    async def connect(self, websocket: WebSocket, channel: str):
        """Establish WebSocket connection"""
        await websocket.accept()
        if channel in self.active_connections:
            self.active_connections[channel].add(websocket)
            self.logger.info(f"New connection registered for channel: {channel}")

    async def disconnect(self, websocket: WebSocket, channel: str):
        """Clean up WebSocket connection"""
        if channel in self.active_connections:
            self.active_connections[channel].discard(websocket)
            self.logger.info(f"Connection removed from channel: {channel}")

    async def broadcast_market_data(self, data: Dict[str, Any]):
        """Broadcast market data to all connected clients"""
        try:
            data["timestamp"] = datetime.now(timezone.utc).isoformat()
            await self._broadcast("market_data", data)
        except Exception as e:
            self.logger.error(f"Market data broadcast failed: {str(e)}")

    async def broadcast_strategy_update(self, strategy_id: str, update: Dict[str, Any]):
        """Broadcast strategy execution updates"""
        try:
            payload = {
                "strategy_id": strategy_id,
                "update": update,
                "timestamp": datetime.now(timezone.utc).isoformat(),
            }
            await self._broadcast("strategy_updates", payload)
        except Exception as e:
            self.logger.error(f"Strategy update broadcast failed: {str(e)}")

    async def _broadcast(self, channel: str, message: Dict[str, Any]):
        """Internal broadcast helper with connection cleanup"""
        if channel not in self.active_connections:
            return

        dead_connections = set()
        for connection in self.active_connections[channel]:
            try:
                await connection.send_json(message)
            except Exception as e:
                self.logger.warning(f"Failed to send to connection: {str(e)}")
                dead_connections.add(connection)

        # Clean up dead connections
        for dead in dead_connections:
            self.active_connections[channel].discard(dead)

    async def start_market_data_stream(self):
        """Initialize market data streaming from Redis"""
        while True:
            try:
                # Read from Redis stream
                data = await self.redis.xread({"market_data": "0-0"}, count=1, block=0)

                if data:
                    # Process and broadcast market data
                    for stream, messages in data:
                        for message_id, fields in messages:
                            await self.broadcast_market_data(
                                json.loads(fields[b"data"].decode())
                            )

            except Exception as e:
                print(f"Market data stream error: {e}")
                await asyncio.sleep(1)  # Backoff on error
