"""
Command-line interface for PineScript converter
"""

import argparse
import json
import sys
from pathlib import Path
from typing import Optional, Dict, Any

from .pine_script import PineScriptConverter, PineScriptConfig, TimeFrame


def parse_config(config_file: Optional[str]) -> Dict[str, Any]:
    """Parse configuration from JSON file"""
    if not config_file:
        return {}

    config_path = Path(config_file)
    if not config_path.exists():
        print(f"Error: Config file {config_file} not found", file=sys.stderr)
        sys.exit(1)

    try:
        with open(config_path) as f:
            return json.load(f)
    except json.JSONDecodeError as e:
        print(f"Error: Invalid JSON in config file: {e}", file=sys.stderr)
        sys.exit(1)


def create_config(config_dict: Dict[str, Any]) -> PineScriptConfig:
    """Create PineScriptConfig from dictionary"""
    # Convert timeframe string to enum
    if "timeframe" in config_dict:
        try:
            config_dict["timeframe"] = TimeFrame(config_dict["timeframe"])
        except ValueError:
            print(
                f"Error: Invalid timeframe {config_dict['timeframe']}", file=sys.stderr
            )
            print(
                f"Valid timeframes: {[tf.value for tf in TimeFrame]}", file=sys.stderr
            )
            sys.exit(1)

    try:
        return PineScriptConfig(**config_dict)
    except TypeError as e:
        print(f"Error: Invalid configuration: {e}", file=sys.stderr)
        sys.exit(1)


def main() -> None:
    """Main entry point for CLI"""
    parser = argparse.ArgumentParser(
        description="Convert PineScript strategies to Python"
    )

    parser.add_argument("input_file", type=str, help="Input PineScript file")

    parser.add_argument(
        "-o",
        "--output",
        type=str,
        help="Output Python file (default: input file with .py extension)",
    )

    parser.add_argument("-c", "--config", type=str, help="JSON configuration file")

    parser.add_argument(
        "-v",
        "--version",
        type=str,
        choices=["4", "5"],
        default="5",
        help="PineScript version (default: 5)",
    )

    parser.add_argument(
        "-t",
        "--timeframe",
        type=str,
        choices=[tf.value for tf in TimeFrame],
        help="Trading timeframe",
    )

    parser.add_argument(
        "--no-overlay", action="store_true", help="Disable chart overlay"
    )

    parser.add_argument("--risk", type=float, help="Risk per trade (0.0-1.0)")

    parser.add_argument(
        "--pyramiding", type=int, help="Maximum number of entries in the same direction"
    )

    args = parser.parse_args()

    # Read input file
    input_path = Path(args.input_file)
    if not input_path.exists():
        print(f"Error: Input file {args.input_file} not found", file=sys.stderr)
        sys.exit(1)

    try:
        with open(input_path) as f:
            pine_script = f.read()
    except Exception as e:
        print(f"Error reading input file: {e}", file=sys.stderr)
        sys.exit(1)

    # Determine output file
    output_path = Path(args.output) if args.output else input_path.with_suffix(".py")

    # Create configuration
    config_dict = parse_config(args.config)

    # Override config with command line arguments
    if args.version:
        config_dict["version"] = args.version
    if args.timeframe:
        config_dict["timeframe"] = args.timeframe
    if args.no_overlay:
        config_dict["overlay"] = False
    if args.risk is not None:
        config_dict["risk_per_trade"] = args.risk
    if args.pyramiding is not None:
        config_dict["pyramiding"] = args.pyramiding

    config = create_config(config_dict)

    # Convert the script
    try:
        converter = PineScriptConverter(config)
        python_code = converter.convert(pine_script)
    except Exception as e:
        print(f"Error converting PineScript: {e}", file=sys.stderr)
        sys.exit(1)

    # Write output
    try:
        with open(output_path, "w") as f:
            f.write(python_code)
        print(f"Successfully converted {input_path} to {output_path}")
    except Exception as e:
        print(f"Error writing output file: {e}", file=sys.stderr)
        sys.exit(1)


if __name__ == "__main__":
    main()
