"""
PineScript converter for translating TradingView strategies to Python
"""

from dataclasses import dataclass, field
from typing import Dict, List, Optional, Union, Tuple
import ast
import re
from enum import Enum


class TimeFrame(Enum):
    """Supported timeframes"""

    TICK = "tick"
    MIN1 = "1"
    MIN5 = "5"
    MIN15 = "15"
    MIN30 = "30"
    HOUR1 = "60"
    HOUR4 = "240"
    DAY = "D"
    WEEK = "W"
    MONTH = "M"


@dataclass
class PineScriptConfig:
    """Configuration for PineScript conversion"""

    version: str = "5"
    timeframe: TimeFrame = TimeFrame.DAY
    overlay: bool = True
    precision: int = 8
    max_bars_back: int = 5000
    additional_imports: List[str] = field(default_factory=list)
    risk_per_trade: float = 0.01
    pyramiding: int = 1


@dataclass
class ConversionContext:
    """Context for the current conversion"""

    indent_level: int = 0
    in_function: bool = False
    current_scope: Dict[str, str] = field(default_factory=dict)
    defined_functions: List[str] = field(default_factory=list)


class PineScriptConverter:
    """Converts PineScript strategies to Python"""

    def __init__(self, config: Optional[PineScriptConfig] = None):
        self.config = config or PineScriptConfig()
        self.context = ConversionContext()
        self._init_conversion_maps()

    def _init_conversion_maps(self) -> None:
        """Initialize conversion mappings"""
        self.builtin_funcs = {
            "sma": "ta.SMA",
            "ema": "ta.EMA",
            "rsi": "ta.RSI",
            "macd": "ta.MACD",
            "crossover": self._convert_crossover,
            "crossunder": self._convert_crossunder,
            "highest": "ta.HIGHEST",
            "lowest": "ta.LOWEST",
            "pivothigh": "ta.PIVOTHIGH",
            "pivotlow": "ta.PIVOTLOW",
            "valuewhen": self._convert_valuewhen,
            "barssince": self._convert_barssince,
            "mom": "ta.MOM",
            "roc": "ta.ROC",
            "stoch": "ta.STOCH",
            "willr": "ta.WILLR",
            "atr": "ta.ATR",
            "bbands": "ta.BBANDS",
            "keltner": self._convert_keltner,
            "obv": "ta.OBV",
            "ad": "ta.AD",
            "adosc": "ta.ADOSC",
            "falling": self._convert_falling,
            "rising": self._convert_rising,
        }

        self.plot_funcs = {
            "plot": self._convert_plot,
            "hline": self._convert_hline,
            "fill": self._convert_fill,
            "bgcolor": self._convert_bgcolor,
        }

        self.strategy_funcs = {
            "entry": self._convert_strategy_entry,
            "exit": self._convert_strategy_exit,
            "close": self._convert_strategy_close,
            "cancel": self._convert_strategy_cancel,
        }

        self.variable_maps = {
            "close": 'df["Close"]',
            "open": 'df["Open"]',
            "high": 'df["High"]',
            "low": 'df["Low"]',
            "volume": 'df["Volume"]',
            "time": "df.index",
            "strategy.equity": "self.equity",
            "strategy.position_size": "self.position_size",
        }

    def convert(self, pine_script: str) -> str:
        """Convert PineScript to Python code

        Args:
            pine_script: PineScript code as string

        Returns:
            Converted Python code as string

        Raises:
            ValueError: If input is invalid or empty
        """
        if not pine_script or not pine_script.strip():
            raise ValueError("Invalid PineScript: Empty input")

        # Split into sections
        sections = self._split_sections(pine_script)
        if not sections:
            raise ValueError("Invalid PineScript: No valid code sections found")

        # Generate imports
        imports = self._generate_imports()

        # Convert each section
        converted_sections = []
        for section in sections:
            converted = self._convert_section(section)
            if converted:
                converted_sections.append(converted)

        if not converted_sections:
            raise ValueError("Invalid PineScript: Failed to convert any sections")

        # Combine all parts
        return "\n\n".join([imports] + converted_sections)

    def _split_sections(self, pine_script: str) -> List[str]:
        """Split PineScript into logical sections"""
        # Remove comments
        clean_script = re.sub(r"//.*$", "", pine_script, flags=re.MULTILINE)

        # Split on major section markers
        sections = re.split(r"//@version|//@strategy|//@function", clean_script)
        return [s.strip() for s in sections if s.strip()]

    def _generate_imports(self) -> str:
        """Generate required Python imports"""
        imports = [
            "import pandas as pd",
            "import numpy as np",
            "import talib as ta",
            "import matplotlib.pyplot as plt",
            "from typing import Dict, List, Optional, Union",
            "from dataclasses import dataclass",
        ]
        imports.extend(self.config.additional_imports)
        return "\n".join(imports)

    def _convert_section(self, section: str) -> Optional[str]:
        """Convert a single section of PineScript"""
        if not section:
            return None

        # Handle different section types
        if section.startswith("strategy"):
            return self._convert_strategy(section)
        elif section.startswith("study"):
            return self._convert_study(section)
        elif "=>" in section:
            return self._convert_function(section)
        else:
            return self._convert_variables(section)

    def _convert_strategy(self, strategy_code: str) -> str:
        """Convert a strategy section"""
        # Extract strategy name and settings
        name_match = re.search(r'strategy\(["\'](.+?)["\']', strategy_code)
        if not name_match:
            raise ValueError("Invalid PineScript: Missing strategy name")

        strategy_name = name_match.group(1)
        # Preserve MA casing exactly
        class_name = (
            strategy_name.replace(" MA ", "MA")
            .replace(" Ma ", "MA")
            .replace(" ma ", "MA")
        )
        class_name = "".join(
            word[0].upper() + word[1:] if word.lower() != "ma" else word
            for word in class_name.split()
        )

        # Convert to Python class
        class_code = [
            f"class {class_name}:",
            "    def __init__(self):",
            "        self.position = 0",
            "        self.position_size = 0",
            "        self.equity = 10000",  # Default starting equity
            "        self.entries = []",
            "        self.exits = []",
            "",
            "    def calculate_signals(self, df: pd.DataFrame) -> Dict[str, pd.Series]:",
            "        # Initialize strategy variables",
            "        position = pd.Series(0, index=df.index)",
            "        entries = pd.Series(False, index=df.index)",
            "        exits = pd.Series(False, index=df.index)",
            "        ",
            "        # Calculate indicators",
        ]

        # Convert strategy logic
        logic = self._convert_strategy_logic(strategy_code)
        class_code.extend(
            ["        " + line for line in logic.split("\n") if line.strip()]
        )

        # Add position tracking
        class_code.extend(
            [
                "        # Update position tracking",
                "        for i in range(len(df)):",
                "            if entries[i] and position[i-1] <= 0:",
                "                position[i:] = 1",
                "            elif exits[i] and position[i-1] > 0:",
                "                position[i:] = 0",
                "        ",
                "        return {",
                "            'position': position,",
                "            'entries': entries,",
                "            'exits': exits",
                "        }",
            ]
        )

        return "\n".join(class_code)

    def _convert_strategy_logic(self, strategy_code: str) -> str:
        """Convert strategy logic to Python"""
        lines = []
        for line in strategy_code.split("\n"):
            line = line.strip()
            if not line or line.startswith("//"):
                continue

            # Skip strategy declaration
            if line.startswith("strategy("):
                continue

            # Convert the line
            converted = self._convert_line(line)
            if converted:
                lines.append(converted)

        return "\n".join(lines)

    def _convert_line(self, line: str) -> str:
        """Convert a single line of PineScript"""
        # Handle if statements
        if line.startswith("if "):
            condition = line[3:].strip()
            return f"if {self._convert_expression(condition)}:"

        # Handle variable assignments
        if "=" in line and "=>" not in line:
            name, value = line.split("=", 1)
            name = name.strip()
            value = value.strip()

            # Handle special assignments
            if name.startswith("strategy."):
                return self._convert_strategy_assignment(name, value)

            # Convert normal assignments
            return f"{name} = {self._convert_expression(value)}"

        # Handle function calls
        return self._convert_expression(line)

    def _convert_strategy_assignment(self, name: str, value: str) -> str:
        """Convert strategy assignments"""
        if name == "strategy.position_size":
            return f"self.position_size = {self._convert_expression(value)}"
        elif name == "strategy.equity":
            return f"self.equity = {self._convert_expression(value)}"
        return f"{name} = {self._convert_expression(value)}"

    def _convert_rising(self, args: List[str]) -> str:
        """Convert rising function"""
        if len(args) != 2:
            raise ValueError("rising() requires source and length arguments")
        source, length = args
        return f"{source}.rolling({length}).apply(lambda x: all(x[i] <= x[i+1] for i in range(len(x)-1)))"

    def _convert_falling(self, args: List[str]) -> str:
        """Convert falling function"""
        if len(args) != 2:
            raise ValueError("falling() requires source and length arguments")
        source, length = args
        return f"{source}.rolling({length}).apply(lambda x: all(x[i] >= x[i+1] for i in range(len(x)-1)))"

    def _convert_expression(self, expr: str) -> Optional[str]:
        """Convert a PineScript expression to Python"""
        if not expr:
            return None

        # Handle variable substitutions
        for pine_var, py_var in self.variable_maps.items():
            expr = expr.replace(pine_var, py_var)

        # Handle function conversions
        for pine_func, py_func in self.builtin_funcs.items():
            if callable(py_func):
                # Handle custom conversion functions
                if pine_func in expr:
                    match = re.search(rf"{pine_func}\((.*?)\)", expr)
                    if match:
                        args = [arg.strip() for arg in match.group(1).split(",")]
                        expr = py_func(args)
            else:
                # Simple string replacement
                expr = expr.replace(pine_func, py_func)

        # Handle plot functions
        for plot_func, plot_converter in self.plot_funcs.items():
            if plot_func in expr:
                match = re.search(rf"{plot_func}\((.*?)\)", expr)
                if match:
                    args = [arg.strip() for arg in match.group(1).split(",")]
                    expr = plot_converter(args)

        # Handle strategy functions
        for strat_func, strat_converter in self.strategy_funcs.items():
            if f"strategy.{strat_func}" in expr:
                match = re.search(rf"strategy\.{strat_func}\((.*?)\)", expr)
                if match:
                    args = [arg.strip() for arg in match.group(1).split(",")]
                    expr = strat_converter(args)

        return expr

    def _convert_crossover(self, args: List[str]) -> str:
        """Convert crossover function"""
        if len(args) != 2:
            raise ValueError("crossover() requires two arguments")
        series1, series2 = args
        return f"ta.crossover({series1}, {series2})"

    def _convert_crossunder(self, args: List[str]) -> str:
        """Convert crossunder function"""
        if len(args) != 2:
            raise ValueError("crossunder() requires two arguments")
        series1, series2 = args
        return f"ta.crossunder({series1}, {series2})"

    def _convert_valuewhen(self, args: List[str]) -> str:
        """Convert valuewhen function"""
        if len(args) != 3:
            raise ValueError(
                "valuewhen() requires condition, source, and occurrence arguments"
            )
        condition, source, occurrence = args
        return f"ta.valuewhen({condition}, {source}, {occurrence})"

    def _convert_barssince(self, args: List[str]) -> str:
        """Convert barssince function"""
        if len(args) != 1:
            raise ValueError("barssince() requires condition argument")
        condition = args[0]
        return f"ta.barssince({condition})"

    def _convert_keltner(self, args: List[str]) -> str:
        """Convert keltner channels"""
        if len(args) != 3:
            raise ValueError(
                "keltner() requires source, length, and multiplier arguments"
            )
        src, length, mult = args
        return f"ta.KELTNER({src}, {length}, {mult})"

    def _convert_plot(self, args: List[str]) -> str:
        """Convert plot function"""
        if len(args) < 1:
            raise ValueError("plot() requires at least series argument")
        series = args[0]
        title = args[1] if len(args) > 1 else '""'
        color = args[2] if len(args) > 2 else '"blue"'
        return f"plt.plot({series}, label={title}, color={color})"

    def _convert_hline(self, args: List[str]) -> str:
        """Convert horizontal line"""
        if len(args) < 1:
            raise ValueError("hline() requires price level argument")
        price = args[0]
        title = args[1] if len(args) > 1 else '""'
        color = args[2] if len(args) > 2 else '"gray"'
        return f"plt.axhline(y={price}, label={title}, color={color})"

    def _convert_fill(self, args: List[str]) -> str:
        """Convert fill between lines"""
        if len(args) < 2:
            raise ValueError("fill() requires two series arguments")
        top, bottom = args[0:2]
        color = args[2] if len(args) > 2 else '"blue"'
        alpha = args[3] if len(args) > 3 else "0.2"
        return (
            f"plt.fill_between(df.index, {top}, {bottom}, color={color}, alpha={alpha})"
        )

    def _convert_bgcolor(self, args: List[str]) -> str:
        """Convert background color"""
        if len(args) < 1:
            raise ValueError("bgcolor() requires color argument")
        color = args[0]
        alpha = args[1] if len(args) > 1 else "0.2"
        return f"plt.axvspan(df.index[0], df.index[-1], color={color}, alpha={alpha})"

    def _convert_strategy_entry(self, args: List[str]) -> str:
        """Convert strategy entry"""
        if len(args) < 2:
            raise ValueError(
                "strategy.entry() requires at least id and direction arguments"
            )
        id_str, direction = args[0:2]
        qty = args[2] if len(args) > 2 else "1.0"
        return f"entries[i] = True if {direction.strip()} == 'long' else False"

    def _convert_strategy_exit(self, args: List[str]) -> str:
        """Convert strategy exit"""
        if len(args) < 1:
            raise ValueError("strategy.exit() requires id argument")
        id_str = args[0]
        return f"exits[i] = True"

    def _convert_strategy_close(self, args: List[str]) -> str:
        """Convert strategy close"""
        if len(args) < 1:
            raise ValueError("strategy.close() requires id argument")
        id_str = args[0]
        return f"exits[i] = True"

    def _convert_strategy_cancel(self, args: List[str]) -> str:
        """Convert strategy cancel"""
        if len(args) < 1:
            raise ValueError("strategy.cancel() requires id argument")
        id_str = args[0]
        return f"entries[i] = False"

    def _convert_function(self, function_code: str) -> str:
        """Convert a PineScript function to Python"""
        # Extract function signature
        match = re.search(r"(\w+)\((.*?)\)\s*=>", function_code)
        if not match:
            raise ValueError("Invalid function definition")

        name, params = match.groups()
        params = [p.strip() for p in params.split(",")]

        # Update context
        self.context.in_function = True
        self.context.defined_functions.append(name)

        # Convert function body
        body_lines = []
        for line in function_code.split("\n"):
            if "=>" not in line:  # Skip function signature
                converted = self._convert_line(line)
                if converted:
                    body_lines.append("    " + converted)

        # Reset context
        self.context.in_function = False

        # Build function definition
        func_def = [
            f'def {name}({", ".join(params)}):',
            *body_lines,
            "    return locals()",  # Return all local variables
        ]

        return "\n".join(func_def)

    def _convert_variables(self, code: str) -> str:
        """Convert PineScript variables to Python"""
        # Handle variable declarations
        lines = []
        for line in code.split("\n"):
            line = line.strip()
            if not line:
                continue

            # Convert variable assignments
            if "=" in line and "=>" not in line:
                var_name, value = line.split("=", 1)
                var_name = var_name.strip()
                value = value.strip()

                # Keep original variable names
                if var_name not in self.variable_maps:
                    self.variable_maps[var_name] = var_name

                # Convert the value
                converted_value = self._convert_expression(value)
                lines.append(f"{var_name} = {converted_value}")

            else:
                lines.append(self._convert_expression(line))

        return "\n".join(lines)

    def _extract_function_params(self, func_code: str) -> Dict[str, str]:
        """Extract function parameters"""
        params = {}
        match = re.search(r"\((.*?)\)", func_code)
        if match:
            param_list = match.group(1).split(",")
            for param in param_list:
                param = param.strip()
                if "=" in param:
                    name, value = param.split("=", 1)
                    params[name.strip()] = value.strip()
                else:
                    params[param] = ""
        return params
