from datetime import datetime
import pandas as pd
import psutil
import torch
import time
from functools import wraps

class ResourceMonitor:
    _instance = None
    
    def __init__(self):
        self.metrics = pd.DataFrame(columns=[
            'timestamp', 'function', 'duration', 
            'memory_mb', 'gpu_memory_mb', 'cpu_percent'
        ])
        
    @classmethod
    def get_instance(cls):
        if not cls._instance:
            cls._instance = ResourceMonitor()
        return cls._instance
    
    def record_metrics(self, **kwargs):
        new_entry = pd.DataFrame([{
            'timestamp': datetime.utcnow(),
            **kwargs
        }])
        self.metrics = pd.concat([self.metrics, new_entry], ignore_index=True)
        
    def get_gpu_memory_mb(self):
        if torch.cuda.is_available():
            return torch.cuda.memory_allocated() / 1024 / 1024
        return 0
        
    def get_system_metrics(self):
        return {
            'memory_mb': psutil.Process().memory_info().rss / 1024 / 1024,
            'cpu_percent': psutil.cpu_percent(),
            'gpu_memory_mb': self.get_gpu_memory_mb()
        }

def track_resource_usage(func):
    @wraps(func)
    def wrapper(*args, **kwargs):
        monitor = ResourceMonitor.get_instance()
        start_time = time.time()
        start_metrics = monitor.get_system_metrics()
        
        try:
            result = func(*args, **kwargs)
            
            # Record metrics
            end_metrics = monitor.get_system_metrics()
            duration = time.time() - start_time
            
            monitor.record_metrics(
                function=func.__name__,
                duration=duration,
                **end_metrics
            )
            
            return result
        except Exception as e:
            # Record metrics even on failure
            end_metrics = monitor.get_system_metrics()
            duration = time.time() - start_time
            
            monitor.record_metrics(
                function=func.__name__,
                duration=duration,
                error=str(e),
                **end_metrics
            )
            raise
    
    return wrapper
