"""Base client for making secure API requests."""

import hmac
import hashlib
import time
import asyncio
from typing import Dict, Optional
import httpx
from datetime import datetime, timezone


class SecureAPIClient:
    """Base class for secure API interactions."""

    def __init__(self, api_key: Optional[str] = None):
        """Initialize client with API key."""
        self.api_key = api_key or "test_key"
        self.last_request_time = 0
        self.min_request_interval = 1.0  # seconds

    def _sign_request(self, endpoint: str, params: Dict) -> str:
        """Sign request with HMAC-SHA256."""
        message = endpoint + "".join(f"{k}{v}" for k, v in sorted(params.items()))
        signature = hmac.new(
            self.api_key.encode(), message.encode(), hashlib.sha256
        ).hexdigest()
        return signature

    async def _get(self, url: str, params: Dict) -> Dict:
        """Make a rate-limited GET request."""
        # Rate limiting
        current_time = time.time()
        time_since_last = current_time - self.last_request_time
        if time_since_last < self.min_request_interval:
            await asyncio.sleep(self.min_request_interval - time_since_last)

        # Add signature
        signature = self._sign_request(url, params)
        params["signature"] = signature

        async with httpx.AsyncClient() as client:
            response = await client.get(url, params=params)
            response.raise_for_status()
            self.last_request_time = time.time()
            return await response.json()

    def _format_timestamp(self, dt: Optional[datetime] = None) -> str:
        """Format timestamp in UTC ISO format."""
        if dt is None:
            dt = datetime.now(timezone.utc)
        return dt.isoformat()
