"""Market data clients for Glassnode and CryptoQuant."""

from typing import Dict, Optional
from datetime import datetime, timedelta, timezone
from .base_client import SecureAPIClient


class GlassnodeClient(SecureAPIClient):
    """Client for Glassnode API."""

    def __init__(self, api_key: Optional[str] = None):
        """Initialize Glassnode client."""
        super().__init__(api_key)
        self.base_url = "https://api.glassnode.com/v1"
        self.cache = {}

    async def get_metrics(self, asset: str = "BTC") -> Dict:
        """Get on-chain metrics from Glassnode."""
        params = {
            "a": asset,
            "api_key": self.api_key,
            "timestamp": self._format_timestamp(),
        }

        metrics = {}

        # Active addresses
        active = await self._get(
            f"{self.base_url}/metrics/addresses/active_count", params
        )
        metrics["active_addresses"] = active.get("value", 0)

        # Exchange flows
        flows = await self._get(
            f"{self.base_url}/metrics/transactions/transfers_volume_exchanges_net",
            params,
        )
        metrics["exchange_flows"] = flows.get("value", 0)

        # SOPR
        sopr = await self._get(f"{self.base_url}/metrics/indicators/sopr", params)
        metrics["sopr"] = sopr.get("value", 0)

        return metrics


class CryptoQuantAPI(SecureAPIClient):
    """Client for CryptoQuant API."""

    def __init__(self, api_key: Optional[str] = None):
        """Initialize CryptoQuant client."""
        super().__init__(api_key)
        self.base_url = "https://api.cryptoquant.com/v1"

    async def get_exchange_flows(self, asset: str = "BTC") -> Dict:
        """Get exchange flow metrics."""
        params = {"asset": asset, "api_key": self.api_key, "window": "24h"}

        response = await self._get(f"{self.base_url}/exchange/flows", params)

        flows = {
            "inflow": response.get("inflow", 0),
            "outflow": response.get("outflow", 0),
        }
        flows["net_flow"] = flows["inflow"] - flows["outflow"]

        return flows

    async def get_miner_data(self, asset: str = "BTC") -> Dict:
        """Get mining pool metrics."""
        params = {"asset": asset, "api_key": self.api_key}

        return await self._get(f"{self.base_url}/mining/pool-flows", params)
