"""News API client for market sentiment analysis."""

from typing import Dict, List, Optional
from datetime import datetime, timedelta, timezone
from .base_client import SecureAPIClient


class NewsAPIClient(SecureAPIClient):
    """Client for NewsAPI."""

    def __init__(self, api_key: Optional[str] = None):
        """Initialize NewsAPI client."""
        super().__init__(api_key)
        self.base_url = "https://newsapi.org/v2"
        self.default_sources = [
            "bloomberg",
            "reuters",
            "financial-times",
            "wall-street-journal",
            "crypto-news-flash",
        ]

    async def get_latest(
        self, keywords: List[str], hours: int = 24, sources: Optional[List[str]] = None
    ) -> List[Dict]:
        """Get latest news articles filtered by keywords."""
        sources = sources or self.default_sources

        params = {
            "q": " OR ".join(keywords),
            "apiKey": self.api_key,
            "sortBy": "publishedAt",
            "language": "en",
            "sources": ",".join(sources),
            "from": (datetime.now(timezone.utc) - timedelta(hours=hours)).isoformat(),
        }

        response = await self._get(f"{self.base_url}/everything", params)

        # Process and clean the articles
        articles = []
        for article in response.get("articles", []):
            articles.append(
                {
                    "title": article.get("title", ""),
                    "description": article.get("description", ""),
                    "url": article.get("url", ""),
                    "published_at": article.get("publishedAt", ""),
                    "source": article.get("source", {}).get("name", ""),
                }
            )

        return articles

    async def get_market_sentiment(self, asset: str) -> Dict:
        """Analyze market sentiment from news articles."""
        articles = await self.get_latest([asset], hours=24)

        # Simple sentiment scoring based on article count and recency
        sentiment_score = min(len(articles) / 10.0, 1.0)  # Normalize to 0-1

        return {
            "sentiment_score": sentiment_score,
            "article_count": len(articles),
            "timestamp": datetime.now(timezone.utc).isoformat(),
        }
