"""Dataset passport for tracking data lineage and quality."""

import hashlib
import json
from datetime import datetime, timezone
from typing import Dict, List, Optional
from pydantic import BaseModel, Field


class DatasetMetrics(BaseModel):
    """Metrics about dataset quality and characteristics."""

    row_count: int
    column_count: int
    missing_values: Dict[str, int]
    data_types: Dict[str, str]
    value_ranges: Dict[str, Dict[str, float]]
    timestamp_range: Dict[str, str]


class DataQualityScore(BaseModel):
    """Quality scores for different aspects of the dataset."""

    completeness: float = Field(ge=0, le=1)
    timeliness: float = Field(ge=0, le=1)
    accuracy: float = Field(ge=0, le=1)
    consistency: float = Field(ge=0, le=1)
    overall: float = Field(ge=0, le=1)


class ProcessingStep(BaseModel):
    """Information about a data processing step."""

    step_id: str
    operation: str
    parameters: Dict
    input_hash: str
    output_hash: str
    timestamp: str


class DatasetPassport(BaseModel):
    """Complete passport for a dataset including lineage and quality metrics."""

    dataset_id: str
    version: str
    created_at: str
    source: str
    description: str
    metrics: DatasetMetrics
    quality_score: DataQualityScore
    processing_history: List[ProcessingStep]
    tags: List[str] = []
    metadata: Dict = {}

    @classmethod
    def create(
        cls,
        df,
        source: str,
        description: str,
        processing_step: Optional[ProcessingStep] = None,
    ) -> "DatasetPassport":
        """Create a new dataset passport from a pandas DataFrame."""
        # Calculate metrics
        metrics = DatasetMetrics(
            row_count=len(df),
            column_count=len(df.columns),
            missing_values=df.isnull().sum().to_dict(),
            data_types={col: str(dtype) for col, dtype in df.dtypes.items()},
            value_ranges={
                col: {"min": float(df[col].min()), "max": float(df[col].max())}
                for col in df.select_dtypes(include=["number"]).columns
            },
            timestamp_range=(
                {"start": df.index.min().isoformat(), "end": df.index.max().isoformat()}
                if isinstance(df.index, pd.DatetimeIndex)
                else {}
            ),
        )

        # Calculate quality scores
        quality_score = DataQualityScore(
            completeness=1 - (df.isnull().sum().sum() / (df.shape[0] * df.shape[1])),
            timeliness=1.0,  # Implement actual timeliness calculation
            accuracy=1.0,  # Implement actual accuracy calculation
            consistency=1.0,  # Implement actual consistency calculation
            overall=0.0,  # Will be calculated below
        )
        quality_score.overall = (
            sum(
                [
                    quality_score.completeness,
                    quality_score.timeliness,
                    quality_score.accuracy,
                    quality_score.consistency,
                ]
            )
            / 4
        )

        # Create processing history
        processing_history = []
        if processing_step:
            processing_history.append(processing_step)

        # Generate dataset ID
        dataset_id = cls._generate_id(source, df)

        return cls(
            dataset_id=dataset_id,
            version=datetime.now(timezone.utc).strftime("%Y%m%d%H%M%S"),
            created_at=datetime.now(timezone.utc).isoformat(),
            source=source,
            description=description,
            metrics=metrics,
            quality_score=quality_score,
            processing_history=processing_history,
        )

    def add_processing_step(
        self, operation: str, parameters: Dict, input_data, output_data
    ) -> None:
        """Add a new processing step to the passport."""
        step = ProcessingStep(
            step_id=f"step_{len(self.processing_history) + 1}",
            operation=operation,
            parameters=parameters,
            input_hash=self._calculate_hash(input_data),
            output_hash=self._calculate_hash(output_data),
            timestamp=datetime.now(timezone.utc).isoformat(),
        )
        self.processing_history.append(step)

    def to_json(self) -> str:
        """Convert passport to JSON string."""
        return self.json(indent=2)

    def save(self, filepath: str) -> None:
        """Save passport to file."""
        with open(filepath, "w") as f:
            f.write(self.to_json())

    @staticmethod
    def load(filepath: str) -> "DatasetPassport":
        """Load passport from file."""
        with open(filepath, "r") as f:
            data = json.load(f)
        return DatasetPassport(**data)

    @staticmethod
    def _generate_id(source: str, data) -> str:
        """Generate unique dataset ID."""
        hash_input = f"{source}_{datetime.now(timezone.utc).isoformat()}"
        return hashlib.sha256(hash_input.encode()).hexdigest()[:12]

    @staticmethod
    def _calculate_hash(data) -> str:
        """Calculate hash of data content."""
        if hasattr(data, "to_json"):
            content = data.to_json()
        elif isinstance(data, (dict, list)):
            content = json.dumps(data, sort_keys=True)
        else:
            content = str(data)
        return hashlib.sha256(content.encode()).hexdigest()
