"""Knowledge graph for research findings and market insights."""

import networkx as nx
from datetime import datetime, timezone
from typing import Dict, List, Optional, Union
import json


class KnowledgeGraph:
    """Maintains a graph of market research findings and relationships."""

    def __init__(self):
        self.graph = nx.DiGraph()
        self.node_types = {
            "MARKET_REGIME": {"color": "blue"},
            "CORRELATION": {"color": "green"},
            "ANOMALY": {"color": "red"},
            "PATTERN": {"color": "purple"},
            "RESEARCH_FINDING": {"color": "orange"},
        }

    def add_finding(
        self, finding_type: str, content: Dict, related_to: Optional[List[str]] = None
    ) -> str:
        """Add a new research finding to the graph."""
        if finding_type not in self.node_types:
            raise ValueError(f"Invalid finding type: {finding_type}")

        # Create unique ID for the finding
        finding_id = (
            f"{finding_type}_{datetime.now(timezone.utc).strftime('%Y%m%d%H%M%S')}"
        )

        # Add node with metadata
        self.graph.add_node(
            finding_id,
            type=finding_type,
            content=content,
            timestamp=datetime.now(timezone.utc).isoformat(),
            **self.node_types[finding_type],
        )

        # Add relationships
        if related_to:
            for related_id in related_to:
                if related_id in self.graph:
                    self.graph.add_edge(
                        finding_id, related_id, relationship="RELATED_TO"
                    )

        return finding_id

    def add_relationship(
        self,
        source_id: str,
        target_id: str,
        relationship_type: str,
        metadata: Optional[Dict] = None,
    ):
        """Add a relationship between findings."""
        if source_id not in self.graph or target_id not in self.graph:
            raise ValueError("Source or target node not found")

        self.graph.add_edge(
            source_id,
            target_id,
            relationship=relationship_type,
            metadata=metadata or {},
            timestamp=datetime.now(timezone.utc).isoformat(),
        )

    def get_related_findings(self, finding_id: str, max_depth: int = 2) -> List[Dict]:
        """Get findings related to a specific finding."""
        if finding_id not in self.graph:
            raise ValueError("Finding not found")

        related = []
        for node in nx.descendants_at_distance(self.graph, finding_id, max_depth):
            related.append(
                {
                    "id": node,
                    "type": self.graph.nodes[node]["type"],
                    "content": self.graph.nodes[node]["content"],
                    "timestamp": self.graph.nodes[node]["timestamp"],
                }
            )

        return related

    def find_patterns(self, pattern_type: str) -> List[Dict]:
        """Find patterns of a specific type in the graph."""
        patterns = []

        if pattern_type == "CORRELATION_CHAIN":
            patterns = self._find_correlation_chains()
        elif pattern_type == "ANOMALY_CLUSTER":
            patterns = self._find_anomaly_clusters()
        elif pattern_type == "REGIME_TRANSITION":
            patterns = self._find_regime_transitions()

        return patterns

    def _find_correlation_chains(self) -> List[Dict]:
        """Find chains of correlated findings."""
        chains = []
        for node in self.graph.nodes():
            if self.graph.nodes[node]["type"] == "CORRELATION":
                path = nx.descendants(self.graph, node)
                if len(path) > 2:  # Minimum chain length
                    chains.append(
                        {"start": node, "path": list(path), "length": len(path)}
                    )
        return chains

    def _find_anomaly_clusters(self) -> List[Dict]:
        """Find clusters of related anomalies."""
        clusters = []
        anomaly_nodes = [
            n for n in self.graph.nodes() if self.graph.nodes[n]["type"] == "ANOMALY"
        ]

        for node in anomaly_nodes:
            cluster = {
                "center": node,
                "members": list(nx.descendants_at_distance(self.graph, node, 1)),
                "size": len(list(nx.descendants_at_distance(self.graph, node, 1))),
            }
            if cluster["size"] > 1:
                clusters.append(cluster)

        return clusters

    def _find_regime_transitions(self) -> List[Dict]:
        """Find market regime transitions."""
        transitions = []
        regime_nodes = [
            n
            for n in self.graph.nodes()
            if self.graph.nodes[n]["type"] == "MARKET_REGIME"
        ]

        for node in regime_nodes:
            successors = list(self.graph.successors(node))
            for succ in successors:
                if self.graph.nodes[succ]["type"] == "MARKET_REGIME":
                    transitions.append(
                        {
                            "from_regime": node,
                            "to_regime": succ,
                            "timestamp": self.graph.nodes[succ]["timestamp"],
                        }
                    )

        return transitions

    def export_graph(self, format: str = "json") -> Union[str, nx.DiGraph]:
        """Export the knowledge graph in specified format."""
        if format == "json":
            return json.dumps(
                {
                    "nodes": [
                        {
                            "id": node,
                            "type": data["type"],
                            "content": data["content"],
                            "timestamp": data["timestamp"],
                        }
                        for node, data in self.graph.nodes(data=True)
                    ],
                    "edges": [
                        {
                            "source": source,
                            "target": target,
                            "relationship": data["relationship"],
                            "metadata": data.get("metadata", {}),
                        }
                        for source, target, data in self.graph.edges(data=True)
                    ],
                }
            )
        elif format == "networkx":
            return self.graph.copy()
        else:
            raise ValueError(f"Unsupported export format: {format}")

    def import_graph(self, data: Union[str, nx.DiGraph]):
        """Import a knowledge graph from exported format."""
        if isinstance(data, str):
            # Import from JSON
            graph_data = json.loads(data)
            self.graph = nx.DiGraph()

            # Add nodes
            for node in graph_data["nodes"]:
                self.graph.add_node(
                    node["id"],
                    type=node["type"],
                    content=node["content"],
                    timestamp=node["timestamp"],
                )

            # Add edges
            for edge in graph_data["edges"]:
                self.graph.add_edge(
                    edge["source"],
                    edge["target"],
                    relationship=edge["relationship"],
                    metadata=edge.get("metadata", {}),
                )

        elif isinstance(data, nx.DiGraph):
            self.graph = data.copy()
        else:
            raise ValueError("Unsupported import format")
