from typing import Dict, List, Any, Optional
from dataclasses import dataclass
import numpy as np
from datetime import datetime
import json

@dataclass
class Parameter:
    name: str
    min_value: float
    max_value: float
    step: float = 0.1
    is_discrete: bool = False
    
    def to_binary_size(self) -> int:
        """Calculate number of binary variables needed to represent parameter."""
        if self.is_discrete:
            return int(np.ceil(np.log2((self.max_value - self.min_value) / self.step + 1)))
        return 8  # Use 8 bits for continuous parameters

@dataclass
class Constraint:
    name: str
    params: List[str]
    strength: float
    constraint_type: str  # 'equality', 'inequality', 'custom'
    
    def to_dict(self) -> Dict:
        return {
            'params': self.params,
            'strength': self.strength,
            'type': self.constraint_type
        }

class StrategySpace:
    """
    Defines the search space for trading strategies using quantum-compatible parameters.
    """
    
    def __init__(self):
        self.parameters: Dict[str, Parameter] = {}
        self.constraints: Dict[str, Constraint] = {}
        self.objectives: Dict[str, float] = {}
        
    def add_parameter(self, 
                     name: str,
                     min_value: float,
                     max_value: float,
                     step: float = 0.1,
                     is_discrete: bool = False) -> None:
        """Add a parameter to the strategy space."""
        self.parameters[name] = Parameter(
            name=name,
            min_value=min_value,
            max_value=max_value,
            step=step,
            is_discrete=is_discrete
        )
        
    def add_constraint(self,
                      name: str,
                      params: List[str],
                      strength: float = 1.0,
                      constraint_type: str = 'equality') -> None:
        """Add a constraint between parameters."""
        self.constraints[name] = Constraint(
            name=name,
            params=params,
            strength=strength,
            constraint_type=constraint_type
        )
        
    def add_objective(self, param: str, weight: float) -> None:
        """Add an objective term for optimization."""
        self.objectives[param] = weight
        
    def to_quantum_format(self) -> Dict[str, Any]:
        """Convert strategy space to quantum-compatible format."""
        return {
            'parameters': {
                name: {
                    'min': param.min_value,
                    'max': param.max_value,
                    'bits': param.to_binary_size(),
                    'discrete': param.is_discrete
                }
                for name, param in self.parameters.items()
            },
            'constraints': {
                name: constraint.to_dict()
                for name, constraint in self.constraints.items()
            },
            'objectives': self.objectives
        }
        
    @classmethod
    def create_default_space(cls) -> 'StrategySpace':
        """Create a default strategy space with common parameters."""
        space = cls()
        
        # Add timeframe parameters
        space.add_parameter('timeframe_entry', 1, 240, 1, True)
        space.add_parameter('timeframe_exit', 1, 240, 1, True)
        
        # Add threshold parameters
        space.add_parameter('threshold_entry', 0, 0.1, 0.001)
        space.add_parameter('threshold_exit', 0, 0.1, 0.001)
        
        # Add position sizing parameters
        space.add_parameter('position_size', 0.01, 1.0, 0.01)
        space.add_parameter('max_positions', 1, 10, 1, True)
        
        # Add constraints
        space.add_constraint(
            'timeframe_order',
            ['timeframe_entry', 'timeframe_exit'],
            strength=2.0,
            constraint_type='inequality'
        )
        
        space.add_constraint(
            'risk_management',
            ['position_size', 'max_positions'],
            strength=1.5,
            constraint_type='custom'
        )
        
        # Add objectives
        space.add_objective('position_size', -0.7)  # Penalize large positions
        space.add_objective('max_positions', -0.3)  # Penalize too many positions
        
        return space
        
    def save(self, filepath: str) -> None:
        """Save strategy space configuration to file."""
        config = {
            'parameters': {
                name: {
                    'min_value': param.min_value,
                    'max_value': param.max_value,
                    'step': param.step,
                    'is_discrete': param.is_discrete
                }
                for name, param in self.parameters.items()
            },
            'constraints': {
                name: {
                    'params': constraint.params,
                    'strength': constraint.strength,
                    'type': constraint.constraint_type
                }
                for name, constraint in self.constraints.items()
            },
            'objectives': self.objectives,
            'metadata': {
                'created_at': datetime.utcnow().isoformat(),
                'version': '1.0.0'
            }
        }
        
        with open(filepath, 'w') as f:
            json.dump(config, f, indent=2)
            
    @classmethod
    def load(cls, filepath: str) -> 'StrategySpace':
        """Load strategy space configuration from file."""
        with open(filepath, 'r') as f:
            config = json.load(f)
            
        space = cls()
        
        # Load parameters
        for name, param_config in config['parameters'].items():
            space.add_parameter(
                name=name,
                min_value=param_config['min_value'],
                max_value=param_config['max_value'],
                step=param_config['step'],
                is_discrete=param_config['is_discrete']
            )
            
        # Load constraints
        for name, constraint_config in config['constraints'].items():
            space.add_constraint(
                name=name,
                params=constraint_config['params'],
                strength=constraint_config['strength'],
                constraint_type=constraint_config['type']
            )
            
        # Load objectives
        for param, weight in config['objectives'].items():
            space.add_objective(param, weight)
            
        return space
