"""
SYS_LLM: Multi-model research and analysis system.
Handles routing between Gemini, DeepSeek, and Sonar models for comprehensive research.
"""

import os
import json
import logging
from datetime import datetime, UTC
from typing import Dict, Any, Optional
from tenacity import retry, stop_after_attempt, wait_exponential, retry_if_not_exception_type
from openai import AsyncOpenAI, OpenAIError

# Configure logging
logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)

MODEL_ROUTING = {
    "initial_research": "google/gemini-2.0-flash-001",       # 1M tokens
    "preliminary_reasoning": "deepseek/deepseek-r1",         # 128k context
    "deep_analysis": "perplexity/sonar-reasoning-online",    # 127k + web
    "code_review": "deepseek/deepseek-r1"                    # Code-specific
}

class ResearchError(Exception):
    """Base class for research-related errors."""
    pass

class InvalidResearchTypeError(ResearchError):
    """Raised when an invalid research type is provided."""
    pass

class ResearchAnalyzer:
    def __init__(self):
        self.client = AsyncOpenAI(
            api_key=os.getenv("OPENAI_API_KEY"),
            base_url="https://openrouter.ai/api/v1"
        )
        self.research_history = []
        
    @retry(
        stop=stop_after_attempt(3),
        wait=wait_exponential(multiplier=1, min=4, max=10),
        retry=retry_if_not_exception_type(InvalidResearchTypeError)
    )
    async def analyze_research(self, content: str, research_type: str) -> Dict[str, Any]:
        """
        Analyze research content using the appropriate model based on research type.
        
        Args:
            content: The research content to analyze
            research_type: Type of research (initial_research, preliminary_reasoning, deep_analysis)
            
        Returns:
            Dict containing analysis results
            
        Raises:
            InvalidResearchTypeError: If the research type is not supported
            ResearchError: If there's an error during analysis
        """
        model = MODEL_ROUTING.get(research_type)
        if not model:
            logger.error(f"Invalid research type: {research_type}")
            raise InvalidResearchTypeError(f"Invalid research type: {research_type}")
            
        logger.info(f"Starting {research_type} analysis using {model}")
        
        try:
            response = await self.client.chat.completions.create(
                model=model,
                messages=[{
                    "role": "system",
                    "content": self._get_system_prompt(research_type)
                }, {
                    "role": "user",
                    "content": content
                }],
                response_format={"type": "json_object"}
            )
            
            result = json.loads(response.choices[0].message.content)
            self._log_research_step(research_type, result)
            return result
            
        except OpenAIError as e:
            logger.error(f"Error in {research_type} analysis: {str(e)}")
            raise ResearchError(f"API error during {research_type} analysis: {str(e)}") from e
        except Exception as e:
            logger.error(f"Unexpected error in {research_type} analysis: {str(e)}")
            raise ResearchError(f"Unexpected error during {research_type} analysis") from e
            
    async def research_workflow(self, content: str) -> Dict[str, Any]:
        """
        Execute the full research workflow using all three models in sequence.
        
        Args:
            content: Initial research content
            
        Returns:
            Dict containing combined analysis results
            
        Raises:
            ResearchError: If there's an error during any stage of the workflow
        """
        try:
            # Stage 1: Broad research with Gemini's 1M context
            logger.info("Starting initial research with Gemini")
            broad_insights = await self.analyze_research(content, "initial_research")
            
            # Stage 2: Preliminary reasoning with DeepSeek
            logger.info("Processing with DeepSeek reasoning")
            reasoned_analysis = await self.analyze_research(
                self._format_for_reasoning(broad_insights),
                "preliminary_reasoning"
            )
            
            # Stage 3: Deep analysis with Sonar
            logger.info("Performing deep analysis with Sonar")
            final_analysis = await self.analyze_research(
                self._format_for_deep_analysis(reasoned_analysis),
                "deep_analysis"
            )
            
            return {
                "timestamp": datetime.now(UTC).isoformat(),
                "broad_insights": broad_insights,
                "reasoned_analysis": reasoned_analysis,
                "final_analysis": final_analysis
            }
            
        except ResearchError as e:
            logger.error(f"Error in research workflow: {str(e)}")
            raise
            
    def _get_system_prompt(self, research_type: str) -> str:
        """Get the appropriate system prompt based on research type."""
        prompts = {
            "initial_research": """You are a research assistant analyzing the latest AI developments.
                               Focus on extracting key innovations, methodologies, and potential applications.
                               Format your response as a structured JSON with sections for innovations, 
                               methodologies, and applications.""",
            "preliminary_reasoning": """You are a reasoning engine analyzing research findings.
                                   Evaluate the implications, identify patterns, and assess potential impact.
                                   Structure your response as a JSON with sections for implications,
                                   patterns, and impact assessment.""",
            "deep_analysis": """You are a deep analysis system with web access.
                           Verify findings, expand on key points, and provide concrete implementation suggestions.
                           Return a JSON with sections for verified_findings, expanded_analysis, and
                           implementation_recommendations."""
        }
        return prompts.get(research_type, "Analyze the provided content.")
        
    def _format_for_reasoning(self, insights: Dict[str, Any]) -> str:
        """Format initial insights for the reasoning stage."""
        return json.dumps({
            "context": "Analyzing AI research findings",
            "initial_insights": insights,
            "reasoning_tasks": [
                "Identify underlying patterns",
                "Assess reliability and impact",
                "Generate hypotheses for further investigation"
            ]
        }, indent=2)
        
    def _format_for_deep_analysis(self, reasoned_data: Dict[str, Any]) -> str:
        """Format reasoned analysis for the deep analysis stage."""
        return json.dumps({
            "context": "Deep analysis of reasoned findings",
            "reasoned_findings": reasoned_data,
            "analysis_requirements": [
                "Verify claims against current research",
                "Identify practical applications",
                "Generate specific implementation guidelines"
            ]
        }, indent=2)
        
    def _log_research_step(self, step: str, result: Dict[str, Any]) -> None:
        """Log research step results for tracking and debugging."""
        self.research_history.append({
            "timestamp": datetime.now(UTC).isoformat(),
            "step": step,
            "result_summary": {k: str(v)[:100] + "..." for k, v in result.items()}
        })
