"""System Core FastAPI Application"""

import os
import sys
import logging
from pathlib import Path
from fastapi import FastAPI, HTTPException, Depends
from fastapi.middleware.cors import CORSMiddleware
from fastapi.security import OAuth2PasswordBearer
import uvicorn
import asyncio
from datetime import datetime, timezone
from typing import Dict, Any, Optional

# Add project root to Python path
project_root = Path(__file__).parent.parent.parent.absolute()
if str(project_root) not in sys.path:
    sys.path.insert(0, str(project_root))

from sys_core.local_host.routers import research, strategy, backtest
from sys_core.local_host.middleware import create_middleware_stack
from sys_core.local_host.security import verify_token, create_access_token
from sys_core.local_host.system_service import serve as grpc_serve
from sys_core.local_host.monitoring import setup_telemetry
from app.core.research_agent import research_agent

# Configure logging
logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)

# Initialize FastAPI with security and monitoring
app = FastAPI(
    title="Research System Core",
    description="Quantitative Research and Backtesting System",
    version="1.0.0",
    middleware=create_middleware_stack()
)

# Configure CORS for DigitalOcean deployment
app.add_middleware(
    CORSMiddleware,
    allow_origins=["*"],  # Update with specific domain in production
    allow_credentials=True,
    allow_methods=["*"],
    allow_headers=["*"],
)

# Security scheme
oauth2_scheme = OAuth2PasswordBearer(tokenUrl="token")

app.include_router(research_agent.router, prefix="/api/v1/research", tags=["research"])

@app.on_event("startup")
async def startup_event():
    """Initialize services on startup"""
    logger.info("Starting Research System Core...")
    try:
        # Initialize telemetry
        setup_telemetry()
        logger.info("Telemetry initialized")
        
        # Start gRPC service
        await grpc_serve()
        logger.info("gRPC service started")
        
    except Exception as e:
        logger.error(f"Startup failed: {e}")
        raise

@app.on_event("shutdown")
async def shutdown_event():
    """Cleanup on shutdown"""
    logger.info("Shutting down Research System Core...")
    try:
        # Add cleanup tasks here
        pass
    except Exception as e:
        logger.error(f"Shutdown error: {e}")

@app.get("/health")
async def health_check():
    """Health check endpoint"""
    return {
        "status": "healthy",
        "timestamp": datetime.now(timezone.utc).isoformat(),
        "version": "1.0.0"
    }

@app.get("/system/info")
async def system_info(token: str = Depends(oauth2_scheme)):
    """Get system information"""
    if not verify_token(token):
        raise HTTPException(status_code=401, detail="Invalid token")
    
    return {
        "version": "1.0.0",
        "environment": os.getenv("ENV", "development"),
        "python_version": sys.version,
        "timestamp": datetime.now(timezone.utc).isoformat()
    }

if __name__ == "__main__":
    port = int(os.environ.get("PORT", 8080))
    config = uvicorn.Config(
        app,
        host="0.0.0.0",
        port=port,
        log_level="info"
    )
    server = uvicorn.Server(config)
    asyncio.run(server.serve())
