from concurrent import futures
import logging
import grpc
from ..api_contracts.v1 import system_pb2_grpc, system_pb2
"""
FastAPI service for system coordination and real-time monitoring.
Handles WebSocket connections and system health checks.
"""
from fastapi import FastAPI, Depends, WebSocket
from .middleware import SecurityMiddleware
from ..gpu.orchestrator import GPUOrchestrator
from ..cost.projector import CostProjector
from ..db.connection import DatabaseConnection
from ..monitor.live_monitor import setup_monitoring
import asyncio
from fastapi.middleware.cors import CORSMiddleware

app = FastAPI()
security = SecurityMiddleware()
db = DatabaseConnection()
gpu_orchestrator = GPUOrchestrator()
cost_projector = CostProjector(gpu_orchestrator)

# Configure CORS for DigitalOcean deployment
app.add_middleware(
    CORSMiddleware,
    allow_origins=["*"],
    allow_credentials=True,
    allow_methods=["*"],
    allow_headers=["*"],
)

@app.on_event("startup")
async def startup_event():
    """Initialize the monitoring system on startup"""
    await setup_monitoring(app)

@app.post("/api/v1/cost/projection")
async def get_cost_projection(
    params: dict,
    _=Depends(security.validate_license)
):
    projections = await cost_projector.project_cost(params)
    return {"projections": projections}

@app.get("/api/v1/system/health")
async def health_check():
    return {"status": "healthy"}

@app.websocket("/ws/code-updates")
async def websocket_endpoint(websocket: WebSocket):
    await websocket.accept()
    try:
        while True:
            data = await websocket.receive_text()
            await websocket.send_text(f"Message received: {data}")
    except Exception as e:
        print(f"WebSocket error: {e}")

class SystemCoordinatorService(system_pb2_grpc.SystemCoordinatorServicer):
    def __init__(self):
        self.registered_backends = {}
        self.auth_tokens = set()

    def RegisterBackend(self, request, context):
        logging.info(f"Registration attempt from {request.backend_id}")
        
        if not self._validate_token(request.auth_token):
            context.set_code(grpc.StatusCode.PERMISSION_DENIED)
            return system_pb2.RegistrationResponse(success=False)
            
        self.registered_backends[request.backend_id] = {
            "capabilities": request.capabilities,
            "auth_token": request.auth_token
        }
        return system_pb2.RegistrationResponse(
            success=True,
            system_token="sys_"+request.auth_token
        )

    def HealthCheck(self, request, context):
        return system_pb2.HealthStatus(
            status=system_pb2.HealthStatus.HEALTHY,
            message="System tier operational"
        )

    def _validate_token(self, token: str) -> bool:
        return token.startswith("mcp_") and len(token) == 32

def serve():
    server = grpc.server(futures.ThreadPoolExecutor(max_workers=10))
    system_pb2_grpc.add_SystemCoordinatorServicer_to_server(
        SystemCoordinatorService(), server
    )
    server.add_insecure_port('[::]:50051')
    server.start()
    return server
