"""Real-time code monitoring for Windsurf IDE integration."""

import os
import sys
import time
import logging
import threading
import torch
from watchdog.observers.polling import PollingObserver as Observer
from watchdog.events import FileSystemEventHandler
from cryptography.fernet import Fernet

# Configure logging
logging.basicConfig(
    level=logging.INFO,
    format='%(asctime)s - %(message)s',
    datefmt='%Y-%m-%d %H:%M:%S',
    handlers=[
        logging.StreamHandler()
    ]
)

logger = logging.getLogger(__name__)

KEY = Fernet.generate_key()
cipher = Fernet(KEY)

RECOVERY_CODES = [
    'f0a645fe8b808860',
    'a5ef35a3cf7d874c',
    '4838fa2c7e30479d',
    '9a35c147daeb0a90',
    '87b580024fe10a18',
    'eaf08ff879600163',
    'cdcd9eb07b867a50',
    'd6b3b7f66072930d'
]

encrypted_codes = cipher.encrypt('|'.join(RECOVERY_CODES).encode())

with open('recovery_codes.enc', 'wb') as f:
    f.write(encrypted_codes)

def verify_recovery_codes():
    try:
        with open('recovery_codes.enc', 'rb') as f:
            encrypted = f.read()
        decrypted = cipher.decrypt(encrypted).decode()
        codes = decrypted.split('|')
        logger.info(f"Successfully verified {len(codes)} recovery codes")
        return True
    except Exception as e:
        logger.error(f"Failed to verify recovery codes: {str(e)}")
        return False

def test_recovery_system():
    """Test recovery code encryption/decryption"""
    try:
        # Read encrypted codes
        with open('recovery_codes.enc', 'rb') as f:
            encrypted = f.read()
        
        # Decrypt and verify format
        decrypted = cipher.decrypt(encrypted).decode()
        codes = decrypted.split('|')
        
        # Verify we have all 8 codes
        if len(codes) != 8:
            logger.error(f"Expected 8 recovery codes, found {len(codes)}")
            return False
            
        # Verify code format (16 hex characters each)
        for code in codes:
            if not (len(code) == 16 and all(c in '0123456789abcdef' for c in code)):
                logger.error(f"Invalid recovery code format: {code}")
                return False
                
        logger.info("Recovery system verification successful")
        return True
        
    except Exception as e:
        logger.error(f"Recovery system test failed: {str(e)}")
        return False

class CodeChangeHandler(FileSystemEventHandler):
    def on_modified(self, event):
        if event.is_directory:
            return
            
        if event.src_path.endswith(('.py', '.md', '.json')):
            logger.info(f"Change detected in: {event.src_path}")

class ResourceMonitor:
    def __init__(self):
        self.gpu_metrics = {
            'memory_usage': [],
            'utilization': []
        }
    
    def track_gpu(self):
        while True:
            if torch.cuda.is_available():
                self.gpu_metrics['memory_usage'].append(
                    torch.cuda.memory_allocated()/torch.cuda.max_memory_allocated()
                )
                self.gpu_metrics['utilization'].append(
                    torch.cuda.utilization()
                )
            time.sleep(1)

def start_monitoring():
    # Get the project root directory
    current_dir = os.path.dirname(os.path.abspath(__file__))
    project_root = os.path.dirname(os.path.dirname(os.path.dirname(current_dir)))
    
    logger.info(f"Starting monitoring in: {project_root}")
    
    if verify_recovery_codes():
        event_handler = CodeChangeHandler()
        observer = Observer()
        observer.schedule(event_handler, project_root, recursive=True)
        observer.start()
        
        monitor = ResourceMonitor()
        threading.Thread(target=monitor.track_gpu, daemon=True).start()
        
        try:
            while True:
                time.sleep(1)
        except KeyboardInterrupt:
            observer.stop()
            observer.join()

if __name__ == "__main__":
    start_monitoring()
