from fastapi import FastAPI, WebSocket
from datetime import datetime
import json
from typing import List, Dict
from pathlib import Path
import asyncio

class CodeMonitor:
    def __init__(self):
        self.active_connections: List[WebSocket] = []
        self.change_history: List[Dict] = []
        
    async def connect(self, websocket: WebSocket):
        await websocket.accept()
        self.active_connections.append(websocket)
        # Send recent history
        for change in self.change_history[-50:]:
            await websocket.send_json(change)
    
    def disconnect(self, websocket: WebSocket):
        self.active_connections.remove(websocket)
    
    async def broadcast_change(self, change_type: str, file_path: str):
        change = {
            "timestamp": datetime.utcnow().isoformat(),
            "type": change_type,
            "file": str(file_path),
            "relative_path": str(Path(file_path).relative_to(Path.cwd()))
        }
        self.change_history.append(change)
        
        # Keep only last 1000 changes
        if len(self.change_history) > 1000:
            self.change_history = self.change_history[-1000:]
            
        # Broadcast to all connected clients
        for connection in self.active_connections:
            try:
                await connection.send_json(change)
            except:
                # Remove dead connections
                self.active_connections.remove(connection)

monitor = CodeMonitor()

async def setup_monitoring(app: FastAPI):
    @app.websocket("/ws/dev/monitor")
    async def websocket_endpoint(websocket: WebSocket):
        await monitor.connect(websocket)
        try:
            while True:
                # Keep connection alive
                await websocket.receive_text()
        except:
            monitor.disconnect(websocket)
            
    @app.post("/api/v1/dev/notify-change")
    async def notify_change(change_type: str, file_path: str):
        await monitor.broadcast_change(change_type, file_path)
        return {"status": "broadcasted"}
        
    @app.get("/api/v1/dev/changes")
    def get_changes():
        return {"changes": monitor.change_history[-50:]}
