"""
SYS_Security: Security module for LLM output validation and sanitization.
"""

import re
import json
import logging
from typing import Dict, Any, List, Optional

logger = logging.getLogger(__name__)

class LLMGuard:
    def __init__(self):
        self.unsafe_patterns = [
            r"subprocess\.run",
            r"os\.system",
            r"eval\(",
            r"exec\(",
            r"pickle\.load",
            r"__import__\(",
            r"globals\(\)",
            r"locals\(\)",
            r"getattr\("
        ]
        
        self.reasoning_safe_patterns = [
            r"hypothesis\s*:",
            r"evidence\s*:",
            r"confidence\s*:\s*\d+%",
            r"impact_assessment\s*:",
            r"limitations\s*:"
        ]
        
    def sanitize_code(self, code: str) -> str:
        """Remove potentially unsafe patterns from code."""
        sanitized = code
        for pattern in self.unsafe_patterns:
            sanitized = re.sub(pattern, "# REMOVED_UNSAFE_CODE", sanitized)
        return sanitized
        
    def validate_reasoning(self, text: str) -> bool:
        """Validate that reasoning output follows expected patterns."""
        return all(re.search(p, text, re.IGNORECASE) for p in self.reasoning_safe_patterns)
        
    def validate_research_output(self, output: Dict[str, Any]) -> Dict[str, Any]:
        """Validate and sanitize research output."""
        try:
            # Ensure required fields are present
            required_fields = ["timestamp", "analysis", "recommendations"]
            if not all(field in output for field in required_fields):
                raise ValueError("Missing required fields in research output")
                
            # Sanitize any code snippets in recommendations
            if "code_examples" in output:
                output["code_examples"] = [
                    self.sanitize_code(code) 
                    for code in output["code_examples"]
                ]
                
            # Validate reasoning patterns if present
            if "reasoning" in output:
                if not self.validate_reasoning(json.dumps(output["reasoning"])):
                    logger.warning("Research output failed reasoning validation")
                    output["reasoning_validated"] = False
                else:
                    output["reasoning_validated"] = True
                    
            return output
            
        except Exception as e:
            logger.error(f"Error validating research output: {str(e)}")
            raise
            
    def scan_for_vulnerabilities(self, content: Dict[str, Any]) -> List[str]:
        """Scan content for potential security vulnerabilities."""
        vulnerabilities = []
        content_str = json.dumps(content)
        
        # Check for potential SQL injection
        if re.search(r"SELECT.*FROM|INSERT.*INTO|UPDATE.*SET|DELETE.*FROM", content_str, re.IGNORECASE):
            vulnerabilities.append("Potential SQL injection detected")
            
        # Check for file system operations
        if re.search(r"open\(|read\(|write\(|file\.", content_str):
            vulnerabilities.append("File system operations detected")
            
        # Check for network operations
        if re.search(r"urllib|requests|http|socket", content_str):
            vulnerabilities.append("Network operations detected")
            
        return vulnerabilities
