"""Test suite for LLM integration module."""

import os
import json
import pytest
from unittest.mock import AsyncMock, patch, MagicMock
from openai.types.chat import ChatCompletion, ChatCompletionMessage
from openai.types.chat.chat_completion import Choice
from sys_core.llm_integration import (
    ResearchAnalyzer,
    ResearchError,
    InvalidResearchTypeError
)

@pytest.fixture
def mock_env_vars():
    """Mock environment variables for testing."""
    with patch.dict(os.environ, {
        "OPENAI_API_KEY": "test_key",
        "OPENROUTER_KEY": "test_key"
    }):
        yield

@pytest.fixture
def mock_chat_completion():
    """Create a mock chat completion response."""
    def _create_completion(content):
        message = ChatCompletionMessage(
            content=json.dumps(content),
            role="assistant",
            function_call=None,
            tool_calls=None
        )
        choice = Choice(
            finish_reason="stop",
            index=0,
            message=message,
            logprobs=None
        )
        return ChatCompletion(
            id="test_completion",
            choices=[choice],
            created=1234567890,
            model="test-model",
            object="chat.completion",
            system_fingerprint=None,
            usage=None
        )
    return _create_completion

@pytest.fixture
async def research_analyzer(mock_env_vars, mock_chat_completion):
    """Create a ResearchAnalyzer instance with mocked OpenAI client."""
    with patch('openai.AsyncOpenAI') as mock_client:
        # Mock the chat completions create method
        mock_create = AsyncMock()
        mock_create.return_value = mock_chat_completion({
            "analysis": "Test analysis",
            "confidence": 0.95
        })
        mock_client.return_value.chat.completions.create = mock_create
        
        # Create analyzer with mocked client
        analyzer = ResearchAnalyzer()
        analyzer.client = mock_client.return_value
        yield analyzer

@pytest.mark.asyncio
async def test_research_workflow(research_analyzer):
    """Test the complete research workflow."""
    test_content = "Test research content"
    result = await research_analyzer.research_workflow(test_content)
    
    assert "timestamp" in result
    assert "broad_insights" in result
    assert "reasoned_analysis" in result
    assert "final_analysis" in result

@pytest.mark.asyncio
async def test_model_routing(research_analyzer):
    """Test if each model type is correctly routed."""
    test_content = "test"
    for research_type in ["initial_research", "preliminary_reasoning", "deep_analysis"]:
        result = await research_analyzer.analyze_research(test_content, research_type)
        assert "analysis" in result
        assert "confidence" in result

@pytest.mark.asyncio
async def test_security_code_sanitization(research_analyzer):
    """Test code sanitization in research results."""
    test_content = """
    def malicious_code():
        import os
        os.system('rm -rf /')
    """
    result = await research_analyzer.analyze_research(test_content, "code_review")
    assert "analysis" in result

@pytest.mark.asyncio
async def test_reasoning_validation(research_analyzer):
    """Test validation of reasoning outputs."""
    test_content = "Test reasoning content"
    result = await research_analyzer.analyze_research(test_content, "preliminary_reasoning")
    assert "analysis" in result

@pytest.mark.asyncio
async def test_vulnerability_scanning(research_analyzer):
    """Test scanning for potential vulnerabilities in content."""
    test_content = "Test content with potential XSS <script>alert('xss')</script>"
    result = await research_analyzer.analyze_research(test_content, "initial_research")
    assert "analysis" in result

@pytest.mark.asyncio
async def test_error_handling(research_analyzer):
    """Test error handling for invalid research types."""
    with pytest.raises(InvalidResearchTypeError):
        await research_analyzer.analyze_research("test", "invalid_type")
