"""
Monte Carlo Simulation Module
Implements advanced Monte Carlo analysis with multiple path generation methods.
"""

import numpy as np
import pandas as pd
from typing import Dict, List, Optional, Tuple
from dataclasses import dataclass, field
from scipy import stats


@dataclass
class MonteCarloConfig:
    """Configuration for Monte Carlo simulation"""

    n_simulations: int = 1000
    confidence_levels: List[float] = field(default_factory=lambda: [0.90, 0.95, 0.99])
    path_generators: List[str] = field(
        default_factory=lambda: ["gaussian", "bootstrap", "historical"]
    )
    include_black_swans: bool = True
    vol_scaling: bool = True


class MonteCarloSimulator:
    """Advanced Monte Carlo simulation with multiple path generation methods"""

    def __init__(self, config: Optional[MonteCarloConfig] = None):
        self.config = config or MonteCarloConfig()
        self.results = {}

    def simulate(self, strategy_data: pd.DataFrame) -> Dict:
        """Run comprehensive Monte Carlo analysis

        Args:
            strategy_data: DataFrame with columns [timestamp, open, high, low, close, volume]

        Returns:
            Dict containing simulation results and risk metrics
        """
        results = {}

        for generator in self.config.path_generators:
            paths = self._generate_paths(strategy_data, generator)
            metrics = self._calculate_path_metrics(paths)
            confidence_intervals = self._calculate_confidence_intervals(metrics)

            results[generator] = {
                "paths": paths,
                "metrics": metrics,
                "confidence_intervals": confidence_intervals,
                "risk_metrics": self._calculate_risk_metrics(paths, metrics),
            }

        if self.config.include_black_swans:
            black_swan_results = self._simulate_black_swans(strategy_data)
            results["black_swan"] = black_swan_results

        self.results = results
        return results

    def _generate_paths(self, data: pd.DataFrame, generator: str) -> np.ndarray:
        """Generate price paths using specified method"""
        returns = np.log(data["close"] / data["close"].shift(1)).dropna()
        n_days = len(returns)
        paths = np.zeros((self.config.n_simulations, n_days))
        initial_price = data["close"].iloc[0]

        if generator == "gaussian":
            # Geometric Brownian Motion
            mu = returns.mean()
            sigma = returns.std()

            for i in range(self.config.n_simulations):
                noise = np.random.normal(mu, sigma, n_days)
                paths[i] = initial_price * np.exp(np.cumsum(noise))

        elif generator == "bootstrap":
            # Historical bootstrap
            for i in range(self.config.n_simulations):
                sampled_returns = returns.sample(n_days, replace=True)
                paths[i] = initial_price * np.exp(np.cumsum(sampled_returns))

        elif generator == "historical":
            # Historical simulation with volatility scaling
            if self.config.vol_scaling:
                target_vol = returns.std()
                rolling_vol = returns.rolling(window=21).std()
                scaled_returns = returns * (target_vol / rolling_vol.fillna(target_vol))

                for i in range(self.config.n_simulations):
                    sampled_returns = scaled_returns.sample(n_days, replace=True)
                    paths[i] = initial_price * np.exp(np.cumsum(sampled_returns))
            else:
                for i in range(self.config.n_simulations):
                    sampled_returns = returns.sample(n_days, replace=True)
                    paths[i] = initial_price * np.exp(np.cumsum(sampled_returns))

        return paths

    def _simulate_black_swans(self, data: pd.DataFrame) -> Dict:
        """Simulate extreme market events"""
        returns = np.log(data["close"] / data["close"].shift(1)).dropna()
        n_days = len(returns)

        # Define extreme events
        events = {
            "market_crash": {"prob": 0.01, "magnitude": -0.20},
            "flash_crash": {"prob": 0.005, "magnitude": -0.10},
            "volatility_spike": {"prob": 0.02, "magnitude": 0.15},
        }

        black_swan_paths = np.zeros((self.config.n_simulations, n_days))

        for i in range(self.config.n_simulations):
            path = np.zeros(n_days)
            current_return = 0

            for day in range(n_days):
                # Check for extreme events
                for event, params in events.items():
                    if np.random.random() < params["prob"]:
                        current_return += params["magnitude"]

                # Add normal market movement
                current_return += np.random.normal(returns.mean(), returns.std())
                path[day] = current_return

            black_swan_paths[i] = np.exp(path).cumprod()

        return {
            "paths": black_swan_paths,
            "metrics": self._calculate_path_metrics(black_swan_paths),
            "events": events,
        }

    def _calculate_path_metrics(self, paths: np.ndarray) -> Dict:
        """Calculate performance metrics for all paths"""
        returns = np.diff(paths, axis=1) / paths[:, :-1]

        metrics = {
            "final_values": paths[:, -1],
            "max_drawdowns": self._calculate_drawdowns(paths),
            "sharpe_ratios": self._calculate_path_sharpe(returns),
            "volatility": returns.std(axis=1) * np.sqrt(252),
            "var": self._calculate_var(returns),
            "cvar": self._calculate_cvar(returns),
        }

        return metrics

    def _calculate_confidence_intervals(self, metrics: Dict) -> Dict:
        """Calculate confidence intervals for metrics"""
        intervals = {}

        for level in self.config.confidence_levels:
            lower_percentile = (1 - level) / 2
            upper_percentile = 1 - lower_percentile

            intervals[f"{level:.2f}"] = {
                metric: np.percentile(
                    values, [lower_percentile * 100, upper_percentile * 100]
                )
                for metric, values in metrics.items()
                if isinstance(values, np.ndarray)
            }

        return intervals

    def _calculate_risk_metrics(self, paths: np.ndarray, metrics: Dict) -> Dict:
        """Calculate comprehensive risk metrics"""
        returns = np.diff(paths, axis=1) / paths[:, :-1]

        risk_metrics = {
            "tail_ratio": self._calculate_tail_ratio(returns),
            "stability_ratio": self._calculate_stability_ratio(returns),
            "calmar_ratio": self._calculate_calmar_ratio(paths, returns),
            "ulcer_index": self._calculate_ulcer_index(paths),
            "pain_ratio": self._calculate_pain_ratio(paths, returns),
        }

        return risk_metrics

    @staticmethod
    def _calculate_drawdowns(paths: np.ndarray) -> np.ndarray:
        """Calculate maximum drawdown for each path"""
        cummax = np.maximum.accumulate(paths, axis=1)
        drawdowns = paths / cummax - 1
        return np.min(drawdowns, axis=1)

    @staticmethod
    def _calculate_path_sharpe(returns: np.ndarray) -> np.ndarray:
        """Calculate Sharpe ratio for each path"""
        return np.sqrt(252) * returns.mean(axis=1) / returns.std(axis=1)

    @staticmethod
    def _calculate_var(returns: np.ndarray, level: float = 0.95) -> float:
        """Calculate Value at Risk"""
        return np.percentile(returns, (1 - level) * 100)

    @staticmethod
    def _calculate_cvar(returns: np.ndarray, level: float = 0.95) -> float:
        """Calculate Conditional Value at Risk"""
        var = np.percentile(returns, (1 - level) * 100)
        return returns[returns <= var].mean()

    @staticmethod
    def _calculate_tail_ratio(returns: np.ndarray) -> float:
        """Calculate ratio of right tail to left tail"""
        right_tail = np.percentile(returns, 95)
        left_tail = np.percentile(returns, 5)
        return abs(right_tail / left_tail)

    @staticmethod
    def _calculate_stability_ratio(returns: np.ndarray) -> float:
        """Calculate return stability ratio"""
        return returns.mean() / returns.std()

    @staticmethod
    def _calculate_calmar_ratio(paths: np.ndarray, returns: np.ndarray) -> float:
        """Calculate Calmar ratio"""
        cagr = (paths[:, -1] / paths[:, 0]) ** (252 / paths.shape[1]) - 1
        max_dd = np.abs(
            np.min(
                np.minimum.accumulate(paths, axis=1)
                / np.maximum.accumulate(paths, axis=1)
                - 1,
                axis=1,
            )
        )
        return cagr.mean() / max_dd.mean()

    @staticmethod
    def _calculate_ulcer_index(paths: np.ndarray) -> float:
        """Calculate Ulcer index"""
        drawdowns = paths / np.maximum.accumulate(paths, axis=1) - 1
        return np.sqrt(np.mean(drawdowns**2))

    @staticmethod
    def _calculate_pain_ratio(paths: np.ndarray, returns: np.ndarray) -> float:
        """Calculate pain ratio"""
        drawdowns = paths / np.maximum.accumulate(paths, axis=1) - 1
        pain_index = np.mean(np.abs(drawdowns))
        return returns.mean() / pain_index
