"""
Walk-Forward Analysis Module
Implements advanced walk-forward testing with anchored and non-anchored modes.
"""

import numpy as np
import pandas as pd
from datetime import datetime, timezone
from typing import Dict, List, Optional, Tuple
from dataclasses import dataclass, field


@dataclass
class WalkForwardConfig:
    """Configuration for walk-forward analysis"""

    lookback_periods: List[int] = field(default_factory=lambda: [100, 200, 500])
    forward_periods: List[int] = field(default_factory=lambda: [20, 50, 100])
    min_trades_per_window: int = 10
    confidence_level: float = 0.95
    require_profitability: bool = True


class WalkForwardAnalyzer:
    """Advanced walk-forward testing with multiple validation modes"""

    def __init__(self, config: Optional[WalkForwardConfig] = None):
        self.config = config or WalkForwardConfig()
        self.results = {}

    def analyze(self, strategy_data: pd.DataFrame) -> Dict:
        """Run comprehensive walk-forward analysis

        Args:
            strategy_data: DataFrame with columns [timestamp, open, high, low, close, volume]

        Returns:
            Dict containing analysis results and stability metrics
        """
        results = {
            "anchored": self._run_anchored_analysis(strategy_data),
            "rolling": self._run_rolling_analysis(strategy_data),
            "metrics": self._calculate_stability_metrics(strategy_data),
        }

        self.results = results
        return results

    def _run_anchored_analysis(self, data: pd.DataFrame) -> Dict:
        """Run anchored walk-forward analysis"""
        results = {}

        # Fixed starting point, expanding window
        start_idx = 0
        for lookback in self.config.lookback_periods:
            for forward in self.config.forward_periods:
                window_results = []

                for i in range(start_idx, len(data) - lookback - forward, forward):
                    train = data.iloc[start_idx : i + lookback]
                    test = data.iloc[i + lookback : i + lookback + forward]

                    # Calculate performance metrics
                    train_metrics = self._calculate_window_metrics(train)
                    test_metrics = self._calculate_window_metrics(test)

                    window_results.append(
                        {
                            "train_period": (train.index[0], train.index[-1]),
                            "test_period": (test.index[0], test.index[-1]),
                            "train_metrics": train_metrics,
                            "test_metrics": test_metrics,
                            "stability_score": self._calculate_stability(
                                train_metrics, test_metrics
                            ),
                        }
                    )

                results[f"lookback_{lookback}_forward_{forward}"] = window_results

        return results

    def _run_rolling_analysis(self, data: pd.DataFrame) -> Dict:
        """Run rolling walk-forward analysis"""
        results = {}

        # Rolling window analysis
        for lookback in self.config.lookback_periods:
            for forward in self.config.forward_periods:
                window_results = []

                for i in range(0, len(data) - lookback - forward, forward):
                    train = data.iloc[i : i + lookback]
                    test = data.iloc[i + lookback : i + lookback + forward]

                    # Calculate performance metrics
                    train_metrics = self._calculate_window_metrics(train)
                    test_metrics = self._calculate_window_metrics(test)

                    window_results.append(
                        {
                            "train_period": (train.index[0], train.index[-1]),
                            "test_period": (test.index[0], test.index[-1]),
                            "train_metrics": train_metrics,
                            "test_metrics": test_metrics,
                            "stability_score": self._calculate_stability(
                                train_metrics, test_metrics
                            ),
                        }
                    )

                results[f"lookback_{lookback}_forward_{forward}"] = window_results

        return results

    def _calculate_window_metrics(self, data: pd.DataFrame) -> Dict:
        """Calculate performance metrics for a given window"""
        returns = data["close"].pct_change().dropna()

        return {
            "sharpe": self._calculate_sharpe(returns),
            "sortino": self._calculate_sortino(returns),
            "max_drawdown": self._calculate_max_drawdown(returns),
            "win_rate": self._calculate_win_rate(returns),
            "profit_factor": self._calculate_profit_factor(returns),
        }

    def _calculate_stability(self, train_metrics: Dict, test_metrics: Dict) -> float:
        """Calculate stability score between train and test periods"""
        weights = {
            "sharpe": 0.3,
            "sortino": 0.2,
            "max_drawdown": 0.2,
            "win_rate": 0.15,
            "profit_factor": 0.15,
        }

        stability_scores = []
        for metric, weight in weights.items():
            train_val = train_metrics[metric]
            test_val = test_metrics[metric]

            # Handle special cases
            if np.isnan(train_val) or np.isnan(test_val):
                stability_scores.append(0.0)
                continue

            if train_val == 0 and test_val == 0:
                stability_scores.append(1.0)
                continue
            elif train_val == 0:
                stability_scores.append(0.0)
                continue

            deviation = abs(1 - (test_val / train_val))
            stability_scores.append(weight * (1 - min(deviation, 1)))

        return sum(stability_scores)

    def _calculate_stability_metrics(self, data: pd.DataFrame) -> Dict:
        """Calculate overall stability metrics"""
        anchored_scores = []
        rolling_scores = []

        for window_type, results in self.results.items():
            if window_type == "anchored":
                for period_results in results.values():
                    scores = [
                        r["stability_score"]
                        for r in period_results
                        if not np.isnan(r["stability_score"])
                    ]
                    if scores:
                        anchored_scores.extend(scores)
            elif window_type == "rolling":
                for period_results in results.values():
                    scores = [
                        r["stability_score"]
                        for r in period_results
                        if not np.isnan(r["stability_score"])
                    ]
                    if scores:
                        rolling_scores.extend(scores)

        # Handle empty results
        if not anchored_scores and not rolling_scores:
            return {
                "anchored_stability": 0.0,
                "rolling_stability": 0.0,
                "overall_stability": 0.0,
                "stability_std": 0.0,
            }

        return {
            "anchored_stability": np.mean(anchored_scores) if anchored_scores else 0.0,
            "rolling_stability": np.mean(rolling_scores) if rolling_scores else 0.0,
            "overall_stability": np.mean(anchored_scores + rolling_scores),
            "stability_std": np.std(anchored_scores + rolling_scores),
        }

    @staticmethod
    def _calculate_sharpe(returns: pd.Series) -> float:
        """Calculate Sharpe ratio"""
        if len(returns) < 2:
            return 0.0
        return np.sqrt(252) * returns.mean() / returns.std()

    @staticmethod
    def _calculate_sortino(returns: pd.Series) -> float:
        """Calculate Sortino ratio"""
        if len(returns) < 2:
            return 0.0
        negative_returns = returns[returns < 0]
        if len(negative_returns) == 0:
            return np.inf
        return np.sqrt(252) * returns.mean() / negative_returns.std()

    @staticmethod
    def _calculate_max_drawdown(returns: pd.Series) -> float:
        """Calculate maximum drawdown"""
        cum_returns = (1 + returns).cumprod()
        rolling_max = cum_returns.expanding(min_periods=1).max()
        drawdowns = cum_returns / rolling_max - 1
        return abs(drawdowns.min())

    @staticmethod
    def _calculate_win_rate(returns: pd.Series) -> float:
        """Calculate win rate"""
        if len(returns) == 0:
            return 0.0
        return len(returns[returns > 0]) / len(returns)

    @staticmethod
    def _calculate_profit_factor(returns: pd.Series) -> float:
        """Calculate profit factor"""
        positive_returns = returns[returns > 0].sum()
        negative_returns = abs(returns[returns < 0].sum())

        if negative_returns == 0:
            return np.inf if positive_returns > 0 else 0

        return positive_returns / negative_returns
